"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const AutoUpdaterService_1 = require("./services/autoUpdater/AutoUpdaterService");
const shim = require('@joplin/lib/shim').default;
const callbackUrlUtils_1 = require("@joplin/lib/callbackUrlUtils");
const electron_1 = require("electron");
const bridge_1 = require("./bridge");
const url = require('url');
const path = require('path');
const { dirname } = require('@joplin/lib/path-utils');
const fs = require('fs-extra');
const electron_2 = require("electron");
const locale_1 = require("@joplin/lib/locale");
const restartInSafeModeFromMain_1 = require("./utils/restartInSafeModeFromMain");
const handleCustomProtocols_1 = require("./utils/customProtocols/handleCustomProtocols");
const timers_1 = require("timers");
const path_1 = require("path");
const reducer_1 = require("@joplin/lib/reducer");
class ElectronAppWrapper {
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    constructor(electronApp, env, profilePath, isDebugMode, initialCallbackUrl) {
        this.logger_ = null;
        this.win_ = null;
        this.mainWindowHidden_ = true;
        this.pluginWindows_ = {};
        this.secondaryWindows_ = new Map();
        this.willQuitApp_ = false;
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        this.tray_ = null;
        this.buildDir_ = null;
        this.rendererProcessQuitReply_ = null;
        this.initialCallbackUrl_ = null;
        this.updaterService_ = null;
        this.customProtocolHandler_ = null;
        this.updatePollInterval_ = null;
        this.startPeriodicUpdateCheck = (updateInterval = AutoUpdaterService_1.defaultUpdateInterval) => {
            this.stopPeriodicUpdateCheck();
            this.updatePollInterval_ = setInterval(() => {
                void this.updaterService_.checkForUpdates(false);
            }, updateInterval);
            (0, timers_1.setTimeout)(this.updaterService_.checkForUpdates, AutoUpdaterService_1.initialUpdateStartup);
        };
        this.stopPeriodicUpdateCheck = () => {
            if (this.updatePollInterval_) {
                clearInterval(this.updatePollInterval_);
                this.updatePollInterval_ = null;
                this.updaterService_ = null;
            }
        };
        this.electronApp_ = electronApp;
        this.env_ = env;
        this.isDebugMode_ = isDebugMode;
        this.profilePath_ = profilePath;
        this.initialCallbackUrl_ = initialCallbackUrl;
    }
    electronApp() {
        return this.electronApp_;
    }
    setLogger(v) {
        this.logger_ = v;
    }
    logger() {
        return this.logger_;
    }
    mainWindow() {
        return this.win_;
    }
    activeWindow() {
        var _a;
        return (_a = electron_1.BrowserWindow.getFocusedWindow()) !== null && _a !== void 0 ? _a : this.win_;
    }
    windowById(joplinId) {
        if (joplinId === reducer_1.defaultWindowId) {
            return this.mainWindow();
        }
        const windowData = this.secondaryWindows_.get(joplinId);
        if (windowData !== undefined) {
            return electron_1.BrowserWindow.fromId(windowData.electronId);
        }
        return null;
    }
    allAppWindows() {
        const allWindowIds = [...this.secondaryWindows_.keys(), reducer_1.defaultWindowId];
        return allWindowIds.map(id => this.windowById(id));
    }
    env() {
        return this.env_;
    }
    initialCallbackUrl() {
        return this.initialCallbackUrl_;
    }
    // Call when the app fails in a significant way.
    //
    // Assumes that the renderer process may be in an invalid state and so cannot
    // be accessed.
    async handleAppFailure(errorMessage, canIgnore, isTesting) {
        await (0, bridge_1.default)().captureException(new Error(errorMessage));
        const buttons = [];
        buttons.push((0, locale_1._)('Quit'));
        const exitIndex = 0;
        if (canIgnore) {
            buttons.push((0, locale_1._)('Ignore'));
        }
        const restartIndex = buttons.length;
        buttons.push((0, locale_1._)('Restart in safe mode'));
        const { response } = await electron_2.dialog.showMessageBox({
            message: (0, locale_1._)('An error occurred: %s', errorMessage),
            buttons,
        });
        if (response === restartIndex) {
            await (0, restartInSafeModeFromMain_1.default)();
            // A hung renderer seems to prevent the process from exiting completely.
            // In this case, crashing the renderer allows the window to close.
            //
            // Also only run this if not testing (crashing the renderer breaks automated
            // tests).
            if (this.win_ && !this.win_.webContents.isCrashed() && !isTesting) {
                this.win_.webContents.forcefullyCrashRenderer();
            }
        }
        else if (response === exitIndex) {
            process.exit(1);
        }
    }
    createWindow() {
        // Set to true to view errors if the application does not start
        const debugEarlyBugs = this.env_ === 'dev' || this.isDebugMode_;
        const windowStateKeeper = require('electron-window-state');
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        const stateOptions = {
            defaultWidth: Math.round(0.8 * electron_1.screen.getPrimaryDisplay().workArea.width),
            defaultHeight: Math.round(0.8 * electron_1.screen.getPrimaryDisplay().workArea.height),
            file: `window-state-${this.env_}.json`,
        };
        if (this.profilePath_)
            stateOptions.path = this.profilePath_;
        // Load the previous state with fallback to defaults
        const windowState = windowStateKeeper(stateOptions);
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        const windowOptions = {
            x: windowState.x,
            y: windowState.y,
            width: windowState.width,
            height: windowState.height,
            minWidth: 100,
            minHeight: 100,
            backgroundColor: '#fff', // required to enable sub pixel rendering, can't be in css
            webPreferences: {
                nodeIntegration: true,
                contextIsolation: false,
                spellcheck: true,
                enableRemoteModule: true,
            },
            webviewTag: true,
            // We start with a hidden window, which is then made visible depending on the showTrayIcon setting
            // https://github.com/laurent22/joplin/issues/2031
            //
            // On Linux/GNOME, however, the window doesn't show correctly if show is false initially:
            // https://github.com/laurent22/joplin/issues/8256
            show: debugEarlyBugs || shim.isGNOME(),
        };
        // Linux icon workaround for bug https://github.com/electron-userland/electron-builder/issues/2098
        // Fix: https://github.com/electron-userland/electron-builder/issues/2269
        if (shim.isLinux())
            windowOptions.icon = path.join(__dirname, '..', 'build/icons/128x128.png');
        this.win_ = new electron_1.BrowserWindow(windowOptions);
        require('@electron/remote/main').enable(this.win_.webContents);
        if (!electron_1.screen.getDisplayMatching(this.win_.getBounds())) {
            const { width: windowWidth, height: windowHeight } = this.win_.getBounds();
            const { width: primaryDisplayWidth, height: primaryDisplayHeight } = electron_1.screen.getPrimaryDisplay().workArea;
            this.win_.setPosition(primaryDisplayWidth / 2 - windowWidth, primaryDisplayHeight / 2 - windowHeight);
        }
        let unresponsiveTimeout = null;
        this.win_.webContents.on('unresponsive', () => {
            // Don't show the "unresponsive" dialog immediately -- the "unresponsive" event
            // can be fired when showing a dialog or modal (e.g. the update dialog).
            //
            // This gives us an opportunity to cancel it.
            if (unresponsiveTimeout === null) {
                const delayMs = 1000;
                unresponsiveTimeout = (0, timers_1.setTimeout)(() => {
                    unresponsiveTimeout = null;
                    void this.handleAppFailure((0, locale_1._)('Window unresponsive.'), true);
                }, delayMs);
            }
        });
        this.win_.webContents.on('responsive', () => {
            if (unresponsiveTimeout !== null) {
                (0, timers_1.clearTimeout)(unresponsiveTimeout);
                unresponsiveTimeout = null;
            }
        });
        this.win_.webContents.on('render-process-gone', async (_event) => {
            await this.handleAppFailure('Renderer process gone.', false);
        });
        this.win_.webContents.on('did-fail-load', async (event) => {
            // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
            if (event.isMainFrame) {
                await this.handleAppFailure('Renderer process failed to load', false);
            }
        });
        this.mainWindowHidden_ = !windowOptions.show;
        this.win_.on('hide', () => {
            this.mainWindowHidden_ = true;
        });
        this.win_.on('show', () => {
            this.mainWindowHidden_ = false;
        });
        void this.win_.loadURL(url.format({
            pathname: path.join(__dirname, 'index.html'),
            protocol: 'file:',
            slashes: true,
        }));
        // Note that on Windows, calling openDevTools() too early results in a white window with no error message.
        // Waiting for one of the ready events might work but they might not be triggered if there's an error, so
        // the easiest is to use a timeout. Keep in mind that if you get a white window on Windows it might be due
        // to this line though.
        if (debugEarlyBugs) {
            (0, timers_1.setTimeout)(() => {
                try {
                    this.win_.webContents.openDevTools();
                }
                catch (error) {
                    // This will throw an exception "Object has been destroyed" if the app is closed
                    // in less that the timeout interval. It can be ignored.
                    console.warn('Error opening dev tools', error);
                }
            }, 3000);
        }
        const addWindowEventHandlers = (webContents) => {
            // will-frame-navigate is fired by clicking on a link within the BrowserWindow.
            webContents.on('will-frame-navigate', event => {
                // If the link changes the URL of the browser window,
                if (event.isMainFrame) {
                    event.preventDefault();
                    void (0, bridge_1.default)().openExternal(event.url);
                }
            });
            // Override calls to window.open and links with target="_blank": Open most in a browser instead
            // of Electron:
            webContents.setWindowOpenHandler((event) => {
                if (event.url === 'about:blank') {
                    // Script-controlled pages: Used for opening notes in new windows
                    return {
                        action: 'allow',
                        overrideBrowserWindowOptions: {
                            webPreferences: {
                                preload: (0, path_1.resolve)(__dirname, './utils/window/secondaryWindowPreload.js'),
                            },
                        },
                    };
                }
                else if (event.url.match(/^https?:\/\//)) {
                    void (0, bridge_1.default)().openExternal(event.url);
                }
                return { action: 'deny' };
            });
            webContents.on('did-create-window', (event) => {
                addWindowEventHandlers(event.webContents);
            });
        };
        addWindowEventHandlers(this.win_.webContents);
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        this.win_.on('close', (event) => {
            // If it's on macOS, the app is completely closed only if the user chooses to close the app (willQuitApp_ will be true)
            // otherwise the window is simply hidden, and will be re-open once the app is "activated" (which happens when the
            // user clicks on the icon in the task bar).
            // On Windows and Linux, the app is closed when the window is closed *except* if the tray icon is used. In which
            // case the app must be explicitly closed with Ctrl+Q or by right-clicking on the tray icon and selecting "Exit".
            let isGoingToExit = false;
            if (process.platform === 'darwin') {
                if (this.willQuitApp_) {
                    isGoingToExit = true;
                }
                else {
                    event.preventDefault();
                    this.hide();
                }
            }
            else {
                const hasBackgroundWindows = this.secondaryWindows_.size > 0;
                if ((hasBackgroundWindows || this.trayShown()) && !this.willQuitApp_) {
                    event.preventDefault();
                    this.win_.hide();
                }
                else {
                    isGoingToExit = true;
                }
            }
            if (isGoingToExit) {
                if (!this.rendererProcessQuitReply_) {
                    // If we haven't notified the renderer process yet, do it now
                    // so that it can tell us if we can really close the app or not.
                    // Search for "appClose" event for closing logic on renderer side.
                    event.preventDefault();
                    if (this.win_)
                        this.win_.webContents.send('appClose');
                }
                else {
                    // If the renderer process has responded, check if we can close or not
                    if (this.rendererProcessQuitReply_.canClose) {
                        // Really quit the app
                        this.rendererProcessQuitReply_ = null;
                        this.win_ = null;
                    }
                    else {
                        // Wait for renderer to finish task
                        event.preventDefault();
                        this.rendererProcessQuitReply_ = null;
                    }
                }
            }
        });
        electron_2.ipcMain.on('secondary-window-added', (event, windowId) => {
            const window = electron_1.BrowserWindow.fromWebContents(event.sender);
            const electronWindowId = window === null || window === void 0 ? void 0 : window.id;
            this.secondaryWindows_.set(windowId, { electronId: electronWindowId });
            // Match the main window's zoom:
            window.webContents.setZoomFactor(this.mainWindow().webContents.getZoomFactor());
            window.once('close', () => {
                this.secondaryWindows_.delete(windowId);
                const allSecondaryWindowsClosed = this.secondaryWindows_.size === 0;
                const mainWindowVisuallyClosed = this.mainWindowHidden_;
                if (allSecondaryWindowsClosed && mainWindowVisuallyClosed && !this.trayShown()) {
                    // Gracefully quit the app if the user has closed all windows
                    this.win_.close();
                }
            });
        });
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        electron_2.ipcMain.on('asynchronous-message', (_event, message, args) => {
            if (message === 'appCloseReply') {
                // We got the response from the renderer process:
                // save the response and try quit again.
                this.rendererProcessQuitReply_ = args;
                this.quit();
            }
        });
        // This handler receives IPC messages from a plugin or from the main window,
        // and forwards it to the main window or the plugin window.
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        electron_2.ipcMain.on('pluginMessage', (_event, message) => {
            try {
                if (message.target === 'mainWindow') {
                    this.win_.webContents.send('pluginMessage', message);
                }
                if (message.target === 'plugin') {
                    const win = this.pluginWindows_[message.pluginId];
                    if (!win) {
                        this.logger().error(`Trying to send IPC message to non-existing plugin window: ${message.pluginId}`);
                        return;
                    }
                    win.webContents.send('pluginMessage', message);
                }
            }
            catch (error) {
                // An error might happen when the app is closing and a plugin
                // sends a message. In which case, the above code would try to
                // access a destroyed webview.
                // https://github.com/laurent22/joplin/issues/4570
                console.error('Could not process plugin message:', message);
                console.error(error);
            }
        });
        electron_2.ipcMain.on('apply-update-now', () => {
            this.updaterService_.updateApp();
        });
        electron_2.ipcMain.on('check-for-updates', () => {
            void this.updaterService_.checkForUpdates(true);
        });
        // Let us register listeners on the window, so we can update the state
        // automatically (the listeners will be removed when the window is closed)
        // and restore the maximized or full screen state
        windowState.manage(this.win_);
        // HACK: Ensure the window is hidden, as `windowState.manage` may make the window
        // visible with isMaximized set to true in window-state-${this.env_}.json.
        // https://github.com/laurent22/joplin/issues/2365
        if (!windowOptions.show) {
            this.win_.hide();
        }
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    registerPluginWindow(pluginId, window) {
        this.pluginWindows_[pluginId] = window;
    }
    async waitForElectronAppReady() {
        if (this.electronApp().isReady())
            return Promise.resolve();
        return new Promise((resolve) => {
            const iid = setInterval(() => {
                if (this.electronApp().isReady()) {
                    clearInterval(iid);
                    resolve(null);
                }
            }, 10);
        });
    }
    quit() {
        this.stopPeriodicUpdateCheck();
        this.electronApp_.quit();
    }
    exit(errorCode = 0) {
        this.electronApp_.exit(errorCode);
    }
    trayShown() {
        return !!this.tray_;
    }
    // This method is used in macOS only to hide the whole app (and not just the main window)
    // including the menu bar. This follows the macOS way of hiding an app.
    hide() {
        this.electronApp_.hide();
    }
    buildDir() {
        if (this.buildDir_)
            return this.buildDir_;
        let dir = `${__dirname}/build`;
        if (!fs.pathExistsSync(dir)) {
            dir = `${dirname(__dirname)}/build`;
            if (!fs.pathExistsSync(dir))
                throw new Error('Cannot find build dir');
        }
        this.buildDir_ = dir;
        return dir;
    }
    trayIconFilename_() {
        let output = '';
        if (process.platform === 'darwin') {
            output = 'macos-16x16Template.png'; // Electron Template Image format
        }
        else {
            output = '16x16.png';
        }
        if (this.env_ === 'dev')
            output = '16x16-dev.png';
        return output;
    }
    // Note: this must be called only after the "ready" event of the app has been dispatched
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    createTray(contextMenu) {
        try {
            this.tray_ = new electron_1.Tray(`${this.buildDir()}/icons/${this.trayIconFilename_()}`);
            this.tray_.setToolTip(this.electronApp_.name);
            this.tray_.setContextMenu(contextMenu);
            this.tray_.on('click', () => {
                if (!this.mainWindow()) {
                    console.warn('The window object was not available during the click event from tray icon');
                    return;
                }
                this.mainWindow().show();
            });
        }
        catch (error) {
            console.error('Cannot create tray', error);
        }
    }
    destroyTray() {
        if (!this.tray_)
            return;
        this.tray_.destroy();
        this.tray_ = null;
    }
    ensureSingleInstance() {
        if (this.env_ === 'dev')
            return false;
        const gotTheLock = this.electronApp_.requestSingleInstanceLock();
        if (!gotTheLock) {
            // Another instance is already running - exit
            this.quit();
            return true;
        }
        // Someone tried to open a second instance - focus our window instead
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        this.electronApp_.on('second-instance', (_e, argv) => {
            const win = this.mainWindow();
            if (!win)
                return;
            if (win.isMinimized())
                win.restore();
            win.show();
            // eslint-disable-next-line no-restricted-properties
            win.focus();
            if (process.platform !== 'darwin') {
                const url = argv.find((arg) => (0, callbackUrlUtils_1.isCallbackUrl)(arg));
                if (url) {
                    void this.openCallbackUrl(url);
                }
            }
        });
        return false;
    }
    initializeCustomProtocolHandler(logger) {
        var _a;
        (_a = this.customProtocolHandler_) !== null && _a !== void 0 ? _a : (this.customProtocolHandler_ = (0, handleCustomProtocols_1.default)(logger));
    }
    // Electron's autoUpdater has to be init from the main process
    initializeAutoUpdaterService(logger, devMode, includePreReleases) {
        if (shim.isWindows() || shim.isMac()) {
            if (!this.updaterService_) {
                this.updaterService_ = new AutoUpdaterService_1.default(this.win_, logger, devMode, includePreReleases);
                this.startPeriodicUpdateCheck();
            }
        }
    }
    getCustomProtocolHandler() {
        return this.customProtocolHandler_;
    }
    async start() {
        // Since we are doing other async things before creating the window, we might miss
        // the "ready" event. So we use the function below to make sure that the app is ready.
        await this.waitForElectronAppReady();
        const alreadyRunning = this.ensureSingleInstance();
        if (alreadyRunning)
            return;
        this.createWindow();
        this.electronApp_.on('before-quit', () => {
            this.willQuitApp_ = true;
        });
        this.electronApp_.on('window-all-closed', () => {
            this.quit();
        });
        this.electronApp_.on('activate', () => {
            this.win_.show();
        });
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        this.electronApp_.on('open-url', (event, url) => {
            event.preventDefault();
            void this.openCallbackUrl(url);
        });
    }
    async openCallbackUrl(url) {
        this.win_.webContents.send('asynchronous-message', 'openCallbackUrl', {
            url: url,
        });
    }
}
exports.default = ElectronAppWrapper;
//# sourceMappingURL=ElectronAppWrapper.js.map