"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const shim_1 = require("@joplin/lib/shim");
const Logger_1 = require("@joplin/utils/Logger");
const locale_1 = require("@joplin/lib/locale");
const bridge_1 = require("./services/bridge");
const KvStore_1 = require("@joplin/lib/services/KvStore");
const ArrayUtils = require("@joplin/lib/ArrayUtils");
const checkForUpdatesUtils_1 = require("./utils/checkForUpdatesUtils");
const compare_versions_1 = require("compare-versions");
const packageInfo = require('./packageInfo.js');
const logger = Logger_1.default.create('checkForUpdates');
let checkInBackground_ = false;
let isCheckingForUpdate_ = false;
function onCheckStarted() {
    logger.info('Starting...');
    isCheckingForUpdate_ = true;
}
function onCheckEnded() {
    logger.info('Done.');
    isCheckingForUpdate_ = false;
}
async function fetchLatestReleases() {
    const response = await shim_1.default.fetch('https://objects.joplinusercontent.com/r/releases');
    if (!response.ok) {
        const responseText = await response.text();
        throw new Error(`Cannot get latest release info: ${responseText.substr(0, 500)}`);
    }
    return (await response.json());
}
function truncateText(text, length) {
    let truncated = text.substring(0, length);
    const lastNewLine = truncated.lastIndexOf('\n');
    // Cut off at a line break unless we'd be cutting off half the text
    if (lastNewLine > length / 2) {
        truncated = `${truncated.substring(0, lastNewLine)}\n...`;
    }
    else {
        truncated = `${truncated.trim()}...`;
    }
    return truncated;
}
async function getSkippedVersions() {
    const r = await KvStore_1.default.instance().value('updateCheck::skippedVersions');
    return r ? JSON.parse(r) : [];
}
async function isSkippedVersion(v) {
    const versions = await getSkippedVersions();
    return versions.includes(v);
}
async function addSkippedVersion(s) {
    let versions = await getSkippedVersions();
    versions.push(s);
    versions = ArrayUtils.unique(versions);
    await KvStore_1.default.instance().setValue('updateCheck::skippedVersions', JSON.stringify(versions));
}
// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
async function checkForUpdates(inBackground, parentWindow, options) {
    if (isCheckingForUpdate_) {
        logger.info('Skipping check because it is already running');
        return;
    }
    onCheckStarted();
    checkInBackground_ = inBackground;
    logger.info(`Checking with options ${JSON.stringify(options)}`);
    try {
        const releases = await fetchLatestReleases();
        const release = (0, checkForUpdatesUtils_1.extractVersionInfo)(releases, process.platform, process.arch, shim_1.default.isPortable(), options);
        logger.info(`Current version: ${packageInfo.version}`);
        logger.info(`Latest version: ${release.version}`);
        logger.info('Is Pre-release:', release.prerelease);
        if ((0, compare_versions_1.compareVersions)(release.version, packageInfo.version) <= 0) {
            if (!checkInBackground_) {
                await (0, bridge_1.default)().showMessageBox((0, locale_1._)('Current version is up-to-date.'));
            }
        }
        else {
            const shouldSkip = checkInBackground_ && await isSkippedVersion(release.version);
            if (shouldSkip) {
                logger.info('Not displaying notification because version has been skipped');
            }
            else {
                const fullReleaseNotes = release.notes.trim() ? `\n\n${release.notes.trim()}` : '';
                const MAX_RELEASE_NOTES_LENGTH = 1000;
                const truncateReleaseNotes = fullReleaseNotes.length > MAX_RELEASE_NOTES_LENGTH;
                const releaseNotes = truncateReleaseNotes ? truncateText(fullReleaseNotes, MAX_RELEASE_NOTES_LENGTH) : fullReleaseNotes;
                const newVersionString = release.prerelease ? (0, locale_1._)('%s (pre-release)', release.version) : release.version;
                const buttonIndex = await (0, bridge_1.default)().showMessageBox(parentWindow, {
                    type: 'info',
                    message: `${(0, locale_1._)('An update is available, do you want to download it now?')}`,
                    detail: `${(0, locale_1._)('Your version: %s', packageInfo.version)}\n${(0, locale_1._)('New version: %s', newVersionString)}${releaseNotes}`,
                    buttons: [(0, locale_1._)('Download'), (0, locale_1._)('Skip this version'), (0, locale_1._)('Full changelog'), (0, locale_1._)('Cancel')],
                    cancelId: 3,
                });
                if (buttonIndex === 0) {
                    void (0, bridge_1.default)().openExternal(release.downloadUrl ? release.downloadUrl : release.pageUrl);
                }
                else if (buttonIndex === 1) {
                    await addSkippedVersion(release.version);
                }
                else if (buttonIndex === 2) {
                    void (0, bridge_1.default)().openExternal('https://joplinapp.org/help/about/changelog/desktop');
                }
            }
        }
    }
    catch (error) {
        logger.error(error);
        if (!checkInBackground_)
            await (0, bridge_1.default)().showErrorMessageBox(error.message);
    }
    finally {
        onCheckEnded();
    }
}
exports.default = checkForUpdates;
//# sourceMappingURL=checkForUpdates.js.map