"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const react_1 = require("react");
const react_dom_1 = require("react-dom");
const focusHandler_1 = require("@joplin/lib/utils/focusHandler");
const useDocument_1 = require("./hooks/useDocument");
const Dialog = props => {
    const [containerElement, setContainerElement] = (0, react_1.useState)(null);
    const containerDocument = (0, useDocument_1.default)(containerElement);
    // For correct focus handling, the dialog element needs to be managed separately from React. In particular,
    // just after creating the dialog, we need to call .showModal() and just **before** closing the dialog, we
    // need to call .close(). This second requirement is particularly difficult, as this needs to happen even
    // if the dialog is closed by removing its parent from the React DOM.
    //
    // Because useEffect cleanup can happen after an element is removed from the HTML DOM, the dialog is managed
    // using native HTML APIs. This allows us to call .close() while the dialog is still attached to the DOM, which
    // allows the browser to restore the focus from before the dialog was opened.
    const dialogElement = useDialogElement(containerDocument, props.onCancel);
    useDialogClassNames(dialogElement, props.className);
    const [contentRendered, setContentRendered] = (0, react_1.useState)(false);
    (0, react_1.useEffect)(() => {
        if (!dialogElement || !contentRendered)
            return;
        if (!dialogElement.open) {
            dialogElement.showModal();
        }
    }, [dialogElement, contentRendered]);
    (0, react_1.useEffect)(() => {
        if (!dialogElement)
            return;
        if (props.contentFillsScreen) {
            dialogElement.classList.add('-fullscreen');
        }
        else {
            dialogElement.classList.remove('-fullscreen');
        }
    }, [props.contentFillsScreen, dialogElement]);
    if (dialogElement && !contentRendered) {
        setContentRendered(true);
    }
    const content = (React.createElement("div", { className: 'content', style: props.contentStyle }, props.children));
    return React.createElement("div", { ref: setContainerElement, className: 'dialog-anchor-node' }, dialogElement && (0, react_dom_1.createPortal)(content, dialogElement));
};
const useDialogElement = (containerDocument, onCancel) => {
    const [dialogElement, setDialogElement] = (0, react_1.useState)(null);
    const onCancelRef = (0, react_1.useRef)(onCancel);
    onCancelRef.current = onCancel;
    (0, react_1.useEffect)(() => {
        if (!containerDocument)
            return () => { };
        const dialog = containerDocument.createElement('dialog');
        dialog.addEventListener('click', event => {
            const onCancel = onCancelRef.current;
            const isBackgroundClick = event.target === dialog;
            if (isBackgroundClick && onCancel) {
                onCancel();
            }
        });
        dialog.classList.add('dialog-modal-layer');
        dialog.addEventListener('cancel', event => {
            const canCancel = !!onCancelRef.current;
            if (!canCancel) {
                // Prevents [Escape] from closing the dialog. In many places, this is handled
                // by external logic.
                // See https://stackoverflow.com/a/61021326
                event.preventDefault();
            }
        });
        const removedReturnValue = 'removed-from-dom';
        dialog.addEventListener('close', () => {
            var _a, _b;
            const closedByCancel = dialog.returnValue !== removedReturnValue;
            if (closedByCancel) {
                (_a = onCancelRef.current) === null || _a === void 0 ? void 0 : _a.call(onCancelRef);
            }
            // Work around what seems to be an Electron bug -- if an input or contenteditable region is refocused after
            // dismissing a dialog, it won't be editable.
            // Note: While this addresses the issue in the note title input, it does not address the issue in the Rich Text Editor.
            if (((_b = containerDocument.activeElement) === null || _b === void 0 ? void 0 : _b.tagName) === 'INPUT') {
                const element = containerDocument.activeElement;
                (0, focusHandler_1.blur)('Dialog', element);
                (0, focusHandler_1.focus)('Dialog', element);
            }
        });
        containerDocument.body.appendChild(dialog);
        setDialogElement(dialog);
        return () => {
            if (dialog.open) {
                // .close: Instructs the browser to restore keyboard focus to whatever was focused
                // before the dialog.
                dialog.close(removedReturnValue);
            }
            dialog.remove();
        };
    }, [containerDocument]);
    return dialogElement;
};
const useDialogClassNames = (dialogElement, classNames) => {
    (0, react_1.useEffect)(() => {
        if (!dialogElement || !classNames) {
            return () => { };
        }
        // The React className prop can include multiple space-separated classes
        const newClassNames = classNames
            .split(/\s+/)
            .filter(name => name.length && !dialogElement.classList.contains(name));
        dialogElement.classList.add(...newClassNames);
        return () => {
            dialogElement.classList.remove(...newClassNames);
        };
    }, [dialogElement, classNames]);
};
exports.default = Dialog;
//# sourceMappingURL=Dialog.js.map