"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const versionInfo_1 = require("@joplin/lib/versionInfo");
const PluginService_1 = require("@joplin/lib/services/plugins/PluginService");
const Setting_1 = require("@joplin/lib/models/Setting");
const restart_1 = require("../services/restart");
const packageInfo = require('../packageInfo.js');
const ipcRenderer = require('electron').ipcRenderer;
class ErrorBoundary extends React.Component {
    constructor() {
        super(...arguments);
        this.state = { error: null, errorInfo: null, pluginInfos: [], plugins: {} };
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    componentDidCatch(error, errorInfo) {
        if (typeof error === 'string')
            error = { message: error };
        const pluginInfos = [];
        let plugins = {};
        try {
            const service = PluginService_1.default.instance();
            plugins = service.plugins;
            const pluginSettings = service.unserializePluginSettings(Setting_1.default.value('plugins.states'));
            for (const pluginId in pluginSettings) {
                const plugin = PluginService_1.default.instance().pluginById(pluginId);
                pluginInfos.push({
                    id: pluginId,
                    name: plugin.manifest.name,
                    enabled: pluginSettings[pluginId].enabled,
                    version: plugin.manifest.version,
                });
            }
        }
        catch (error) {
            console.error('Could not get plugin info:', error);
        }
        this.setState({ error, errorInfo, pluginInfos, plugins });
    }
    componentDidMount() {
        const onAppClose = () => {
            ipcRenderer.send('asynchronous-message', 'appCloseReply', {
                canClose: true,
            });
        };
        ipcRenderer.on('appClose', onAppClose);
    }
    renderMessage() {
        const message = this.props.message || 'Joplin encountered a fatal error and could not continue.';
        return React.createElement("p", null, message);
    }
    render() {
        if (this.state.error) {
            const safeMode_click = async () => {
                Setting_1.default.setValue('isSafeMode', true);
                await Setting_1.default.saveAll();
                await (0, restart_1.default)();
            };
            try {
                const output = [];
                output.push(React.createElement("section", { key: "message" },
                    React.createElement("h2", null, "Message"),
                    React.createElement("p", null, this.state.error.message)));
                output.push(React.createElement("section", { key: "versionInfo" },
                    React.createElement("h2", null, "Version info"),
                    React.createElement("pre", null, (0, versionInfo_1.default)(packageInfo, this.state.plugins).message)));
                if (this.state.pluginInfos.length) {
                    output.push(React.createElement("section", { key: "pluginSettings" },
                        React.createElement("h2", null, "Plugins"),
                        React.createElement("pre", null, JSON.stringify(this.state.pluginInfos, null, 4))));
                }
                if (this.state.error.stack) {
                    output.push(React.createElement("section", { key: "stacktrace" },
                        React.createElement("h2", null, "Stack trace"),
                        React.createElement("pre", null, this.state.error.stack)));
                }
                if (this.state.errorInfo) {
                    if (this.state.errorInfo.componentStack) {
                        output.push(React.createElement("section", { key: "componentStack" },
                            React.createElement("h2", null, "Component stack"),
                            React.createElement("pre", null, this.state.errorInfo.componentStack)));
                    }
                }
                return (React.createElement("div", { style: { overflow: 'auto', fontFamily: 'sans-serif', padding: '5px 20px' } },
                    React.createElement("h1", null, "Error"),
                    this.renderMessage(),
                    React.createElement("p", null, "To report the error, please copy the *entire content* of this page and post it on Joplin forum or GitHub."),
                    React.createElement("p", null,
                        "If the error persists you may try to ",
                        React.createElement("a", { href: "#", onClick: safeMode_click }, "restart in safe mode"),
                        ", which will temporarily disable all plugins."),
                    output));
            }
            catch (error) {
                return (React.createElement("div", null, JSON.stringify(this.state)));
            }
        }
        return this.props.children;
    }
}
exports.default = ErrorBoundary;
//# sourceMappingURL=ErrorBoundary.js.map