"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
const locale_1 = require("@joplin/lib/locale");
const types_1 = require("@joplin/lib/services/plugins/api/types");
const MenuUtils_1 = require("@joplin/lib/services/commands/MenuUtils");
const CommandService_1 = require("@joplin/lib/services/CommandService");
const SpellCheckerService_1 = require("@joplin/lib/services/spellChecker/SpellCheckerService");
const eventManager_1 = require("@joplin/lib/eventManager");
const bridge_1 = require("../../../../../services/bridge");
const Setting_1 = require("@joplin/lib/models/Setting");
const Menu = (0, bridge_1.default)().Menu;
const MenuItem = (0, bridge_1.default)().MenuItem;
const menuUtils = new MenuUtils_1.default(CommandService_1.default.instance());
const useContextMenu = (props) => {
    const editorRef = props.editorRef;
    // The below code adds support for spellchecking when it is enabled
    // It might be buggy, refer to the below issue
    // https://github.com/laurent22/joplin/pull/3974#issuecomment-718936703
    (0, react_1.useEffect)(() => {
        const isAncestorOfCodeMirrorEditor = (elem) => {
            for (; elem.parentElement; elem = elem.parentElement) {
                if (elem.classList.contains(props.editorClassName)) {
                    return true;
                }
            }
            return false;
        };
        const convertFromScreenCoordinates = (zoomPercent, screenXY) => {
            const zoomFraction = zoomPercent / 100;
            return screenXY / zoomFraction;
        };
        function pointerInsideEditor(params) {
            var _a;
            const x = params.x, y = params.y, isEditable = params.isEditable;
            const containerDoc = (_a = props.containerRef.current) === null || _a === void 0 ? void 0 : _a.ownerDocument;
            const elements = containerDoc === null || containerDoc === void 0 ? void 0 : containerDoc.getElementsByClassName(props.editorClassName);
            // Note: We can't check inputFieldType here. When spellcheck is enabled,
            // params.inputFieldType is "none". When spellcheck is disabled,
            // params.inputFieldType is "plainText". Thus, such a check would be inconsistent.
            if (!(elements === null || elements === void 0 ? void 0 : elements.length) || !isEditable)
                return false;
            // Checks whether the element the pointer clicked on is inside the editor.
            // This logic will need to be changed if the editor is eventually wrapped
            // in an iframe, as elementFromPoint will return the iframe container (and not
            // a child of the editor).
            const zoom = Setting_1.default.value('windowContentZoomFactor');
            const xScreen = convertFromScreenCoordinates(zoom, x);
            const yScreen = convertFromScreenCoordinates(zoom, y);
            const intersectingElement = containerDoc.elementFromPoint(xScreen, yScreen);
            return intersectingElement && isAncestorOfCodeMirrorEditor(intersectingElement);
        }
        async function onContextMenu(event, params) {
            if (!pointerInsideEditor(params))
                return;
            // Don't show the default menu.
            event.preventDefault();
            const menu = new Menu();
            const hasSelectedText = editorRef.current && !!editorRef.current.getSelection();
            menu.append(new MenuItem({
                label: (0, locale_1._)('Cut'),
                enabled: hasSelectedText,
                click: async () => {
                    props.editorCutText();
                },
            }));
            menu.append(new MenuItem({
                label: (0, locale_1._)('Copy'),
                enabled: hasSelectedText,
                click: async () => {
                    props.editorCopyText();
                },
            }));
            menu.append(new MenuItem({
                label: (0, locale_1._)('Paste'),
                enabled: true,
                click: async () => {
                    props.editorPaste();
                },
            }));
            const spellCheckerMenuItems = SpellCheckerService_1.default.instance().contextMenuItems(params.misspelledWord, params.dictionarySuggestions);
            for (const item of spellCheckerMenuItems) {
                menu.append(new MenuItem(item));
            }
            // CodeMirror 5 only:
            // Typically CodeMirror handles all interactions itself (highlighting etc.)
            // But in the case of clicking a misspelled word, we need electron to handle the click
            // The result is that CodeMirror doesn't know what's been selected and doesn't
            // move the cursor into the correct location.
            // and when the user selects a new spelling it will be inserted in the wrong location
            // So in this situation, we use must manually align the internal codemirror selection
            // to the contextmenu selection
            if (editorRef.current && !editorRef.current.cm6 && spellCheckerMenuItems.length > 0) {
                // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
                editorRef.current.alignSelection(params);
            }
            let filterObject = {
                items: [],
            };
            filterObject = await eventManager_1.default.filterEmit('editorContextMenu', filterObject);
            for (const item of filterObject.items) {
                menu.append(new MenuItem({
                    label: item.label,
                    click: async () => {
                        const args = item.commandArgs || [];
                        void CommandService_1.default.instance().execute(item.commandName, ...args);
                    },
                    type: item.type,
                }));
            }
            // eslint-disable-next-line github/array-foreach, @typescript-eslint/no-explicit-any -- Old code before rule was applied, Old code before rule was applied
            menuUtils.pluginContextMenuItems(props.plugins, types_1.MenuItemLocation.EditorContextMenu).forEach((item) => {
                menu.append(new MenuItem(item));
            });
            menu.popup({ window: (0, bridge_1.default)().activeWindow() });
        }
        // Prepend the event listener so that it gets called before
        // the listener that shows the default menu.
        const targetWindow = (0, bridge_1.default)().activeWindow();
        targetWindow.webContents.prependListener('context-menu', onContextMenu);
        return () => {
            if (!targetWindow.isDestroyed()) {
                targetWindow.webContents.off('context-menu', onContextMenu);
            }
        };
    }, [
        props.plugins, props.editorClassName, editorRef, props.containerRef,
        props.editorCutText, props.editorCopyText, props.editorPaste,
    ]);
};
exports.default = useContextMenu;
//# sourceMappingURL=useContextMenu.js.map