"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.svgUriToPng = exports.svgDimensions = exports.textToDataUri = exports.resourceInfo = exports.ContextMenuItemType = void 0;
const Resource_1 = require("@joplin/lib/models/Resource");
const Logger_1 = require("@joplin/utils/Logger");
const logger = Logger_1.default.create('contextMenuUtils');
var ContextMenuItemType;
(function (ContextMenuItemType) {
    ContextMenuItemType["None"] = "";
    ContextMenuItemType["Image"] = "image";
    ContextMenuItemType["Resource"] = "resource";
    ContextMenuItemType["Text"] = "text";
    ContextMenuItemType["Link"] = "link";
})(ContextMenuItemType || (exports.ContextMenuItemType = ContextMenuItemType = {}));
async function resourceInfo(options) {
    const resource = options.resourceId ? await Resource_1.default.load(options.resourceId) : null;
    const resourcePath = resource ? Resource_1.default.fullPath(resource) : null;
    const filename = resource ? (resource.filename ? resource.filename : resource.title) : options.filename ? options.filename : '';
    return { resource, resourcePath, filename };
}
exports.resourceInfo = resourceInfo;
function textToDataUri(text, mime) {
    return `data:${mime};base64,${Buffer.from(text).toString('base64')}`;
}
exports.textToDataUri = textToDataUri;
const svgDimensions = (document, svg) => {
    let width;
    let height;
    try {
        const parser = new DOMParser();
        const id = parser.parseFromString(svg, 'text/html').querySelector('svg').id;
        ({ width, height } = document.querySelector('.noteTextViewer').contentWindow.document.querySelector(`#${id}`).getBoundingClientRect());
    }
    catch (error) {
        logger.warn('Could not get SVG dimensions.');
        logger.warn('Error was: ', error);
    }
    if (!width || !height) {
        return [undefined, undefined];
    }
    return [width, height];
};
exports.svgDimensions = svgDimensions;
const svgUriToPng = (document, svg, width, height) => {
    return new Promise((resolve, reject) => {
        let canvas;
        let img;
        const cleanUpAndReject = (e) => {
            if (canvas)
                canvas.remove();
            if (img)
                img.remove();
            return reject(e);
        };
        try {
            img = document.createElement('img');
            if (!img)
                throw new Error('Failed to create img element');
        }
        catch (e) {
            return cleanUpAndReject(e);
        }
        img.onload = function () {
            try {
                canvas = document.createElement('canvas');
                if (!canvas)
                    throw new Error('Failed to create canvas element');
                if (!width || !height) {
                    const maxDimension = 1024;
                    if (img.width > img.height) {
                        width = maxDimension;
                        height = width * (img.height / img.width);
                    }
                    else {
                        height = maxDimension;
                        width = height * (img.width / img.height);
                    }
                }
                canvas.width = width;
                canvas.height = height;
                const ctx = canvas.getContext('2d');
                if (!ctx)
                    throw new Error('Failed to get context');
                ctx.drawImage(img, 0, 0, canvas.width, canvas.height, 0, 0, canvas.width, canvas.height);
                const pngUri = canvas.toDataURL('image/png');
                if (!pngUri)
                    throw new Error('Failed to generate png uri');
                const pngBase64 = pngUri.split(',')[1];
                const byteString = atob(pngBase64);
                // write the bytes of the string to a typed array
                const buff = new Uint8Array(byteString.length);
                for (let i = 0; i < byteString.length; i++) {
                    buff[i] = byteString.charCodeAt(i);
                }
                canvas.remove();
                img.remove();
                resolve(buff);
            }
            catch (error) {
                cleanUpAndReject(error);
            }
        };
        img.onerror = function (e) {
            cleanUpAndReject(new Error(e.toString()));
        };
        img.src = svg;
    });
};
exports.svgUriToPng = svgUriToPng;
//# sourceMappingURL=contextMenuUtils.js.map