"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.processPastedHtml = exports.getResourcesFromPasteEvent = exports.resourcesStatus = exports.commandAttachFileToBody = exports.handleResourceDownloadMode = void 0;
const shim_1 = require("@joplin/lib/shim");
const Setting_1 = require("@joplin/lib/models/Setting");
const Note_1 = require("@joplin/lib/models/Note");
const Resource_1 = require("@joplin/lib/models/Resource");
const bridge = require('@electron/remote').require('./bridge').default;
const ResourceFetcher_1 = require("@joplin/lib/services/ResourceFetcher");
const htmlUtils_1 = require("@joplin/lib/htmlUtils");
const htmlUtils_2 = require("@joplin/renderer/htmlUtils");
const Logger_1 = require("@joplin/utils/Logger");
const url_1 = require("@joplin/utils/url");
const renderer_1 = require("@joplin/renderer");
const markupRenderOptions_1 = require("./markupRenderOptions");
const path_1 = require("@joplin/utils/path");
const joplinRendererUtils = require('@joplin/renderer').utils;
const { clipboard } = require('electron');
const mimeUtils = require("@joplin/lib/mime-utils");
const md5 = require('md5');
const path = require('path');
const logger = Logger_1.default.create('resourceHandling');
async function handleResourceDownloadMode(noteBody) {
    if (noteBody && Setting_1.default.value('sync.resourceDownloadMode') === 'auto') {
        const resourceIds = await Note_1.default.linkedResourceIds(noteBody);
        await ResourceFetcher_1.default.instance().markForDownload(resourceIds);
    }
}
exports.handleResourceDownloadMode = handleResourceDownloadMode;
// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
async function commandAttachFileToBody(body, filePaths = null, options = null) {
    options = Object.assign({ createFileURL: false, position: 0, markupLanguage: renderer_1.MarkupLanguage.Markdown }, options);
    if (!filePaths) {
        filePaths = await bridge().showOpenDialog({
            properties: ['openFile', 'createDirectory', 'multiSelections'],
        });
        if (!filePaths || !filePaths.length)
            return null;
    }
    for (let i = 0; i < filePaths.length; i++) {
        const filePath = filePaths[i];
        try {
            logger.info(`Attaching ${filePath}`);
            const newBody = await shim_1.default.attachFileToNoteBody(body, filePath, options.position, {
                createFileURL: options.createFileURL,
                resizeLargeImages: Setting_1.default.value('imageResizing'),
                markupLanguage: options.markupLanguage,
            });
            if (!newBody) {
                logger.info('File attachment was cancelled');
                return null;
            }
            body = newBody;
            logger.info('File was attached.');
        }
        catch (error) {
            logger.error(error);
            bridge().showErrorMessageBox(error.message);
        }
    }
    return body;
}
exports.commandAttachFileToBody = commandAttachFileToBody;
// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
function resourcesStatus(resourceInfos) {
    let lowestIndex = joplinRendererUtils.resourceStatusIndex('ready');
    for (const id in resourceInfos) {
        const s = joplinRendererUtils.resourceStatus(Resource_1.default, resourceInfos[id]);
        const idx = joplinRendererUtils.resourceStatusIndex(s);
        if (idx < lowestIndex)
            lowestIndex = idx;
    }
    return joplinRendererUtils.resourceStatusName(lowestIndex);
}
exports.resourcesStatus = resourcesStatus;
// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
async function getResourcesFromPasteEvent(event) {
    const output = [];
    const formats = clipboard.availableFormats();
    for (let i = 0; i < formats.length; i++) {
        const format = formats[i].toLowerCase();
        const formatType = format.split('/')[0];
        if (formatType === 'image') {
            if (event)
                event.preventDefault();
            const image = clipboard.readImage();
            const fileExt = mimeUtils.toFileExtension(format);
            const filePath = `${Setting_1.default.value('tempDir')}/${md5(Date.now())}.${fileExt}`;
            await shim_1.default.writeImageToFile(image, format, filePath);
            const md = await commandAttachFileToBody('', [filePath]);
            await shim_1.default.fsDriver().remove(filePath);
            if (md)
                output.push(md);
        }
    }
    return output;
}
exports.getResourcesFromPasteEvent = getResourcesFromPasteEvent;
const processImagesInPastedHtml = async (html) => {
    const allImageUrls = [];
    const mappedResources = {};
    htmlUtils_1.default.replaceImageUrls(html, (src) => {
        allImageUrls.push(src);
    });
    const downloadImage = async (imageSrc) => {
        try {
            const fileExt = (0, path_1.safeFileExtension)((0, path_1.fileExtension)(imageSrc));
            const name = (0, path_1.safeFilename)((0, path_1.filename)(imageSrc));
            const pieces = [name ? name : md5(Date.now() + Math.random())];
            if (fileExt)
                pieces.push(fileExt);
            const filePath = `${Setting_1.default.value('tempDir')}/${pieces.join('.')}`;
            await shim_1.default.fetchBlob(imageSrc, { path: filePath });
            const createdResource = await shim_1.default.createResourceFromPath(filePath);
            await shim_1.default.fsDriver().remove(filePath);
            mappedResources[imageSrc] = `file://${encodeURI(Resource_1.default.fullPath(createdResource))}`;
        }
        catch (error) {
            logger.warn(`Error creating a resource for ${imageSrc}.`, error);
            mappedResources[imageSrc] = imageSrc;
        }
    };
    const downloadImages = [];
    for (const imageSrc of allImageUrls) {
        if (!mappedResources[imageSrc]) {
            logger.info(`processPastedHtml: Processing image ${imageSrc}`);
            try {
                if (imageSrc.startsWith('file')) {
                    const imageFilePath = path.normalize((0, url_1.fileUriToPath)(imageSrc));
                    const resourceDirPath = path.normalize(Setting_1.default.value('resourceDir'));
                    if (imageFilePath.startsWith(resourceDirPath)) {
                        mappedResources[imageSrc] = imageSrc;
                    }
                    else {
                        const createdResource = await shim_1.default.createResourceFromPath(imageFilePath);
                        mappedResources[imageSrc] = `file://${encodeURI(Resource_1.default.fullPath(createdResource))}`;
                    }
                }
                else if (imageSrc.startsWith('data:')) {
                    mappedResources[imageSrc] = imageSrc;
                }
                else {
                    downloadImages.push(downloadImage(imageSrc));
                }
            }
            catch (error) {
                logger.warn(`processPastedHtml: Error creating a resource for ${imageSrc}.`, error);
                mappedResources[imageSrc] = imageSrc;
            }
        }
    }
    await Promise.all(downloadImages);
    return htmlUtils_1.default.replaceImageUrls(html, (src) => mappedResources[src]);
};
async function processPastedHtml(html, htmlToMd, mdToHtml) {
    // When copying text from eg. GitHub, the HTML might contain non-breaking
    // spaces instead of regular spaces. If these non-breaking spaces are
    // inserted into the TinyMCE editor (using insertContent), they will be
    // dropped. So here we convert them to regular spaces.
    // https://stackoverflow.com/a/31790544/561309
    html = html.replace(/[\u202F\u00A0]/g, ' ');
    html = await processImagesInPastedHtml(html);
    // TinyMCE can accept any type of HTML, including HTML that may not be preserved once saved as
    // Markdown. For example the content may have a dark background which would be supported by
    // TinyMCE, but lost once the note is saved. So here we convert the HTML to Markdown then back
    // to HTML to ensure that the content we paste will be handled correctly by the app.
    if (htmlToMd && mdToHtml) {
        const md = await htmlToMd(renderer_1.MarkupLanguage.Markdown, html, '', { preserveColorStyles: Setting_1.default.value('editor.pastePreserveColors') });
        html = (await mdToHtml(renderer_1.MarkupLanguage.Markdown, md, (0, markupRenderOptions_1.default)({ bodyOnly: true }))).html;
        // When plugins that add to the end of rendered content are installed, bodyOnly can
        // fail to remove the wrapping paragraph. This works around that issue by removing
        // the wrapping paragraph in more cases. See issue #10061.
        if (!md.trim().includes('\n')) {
            html = (0, htmlUtils_2.removeWrappingParagraphAndTrailingEmptyElements)(html);
        }
    }
    return (0, htmlUtils_2.extractHtmlBody)(htmlUtils_2.default.sanitizeHtml(html, {
        allowedFilePrefixes: [Setting_1.default.value('resourceDir')],
    }));
}
exports.processPastedHtml = processPastedHtml;
//# sourceMappingURL=resourceHandling.js.map