"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
const types_1 = require("./types");
const AsyncActionQueue_1 = require("@joplin/lib/AsyncActionQueue");
const resourceHandling_1 = require("./resourceHandling");
const HtmlToHtml_1 = require("@joplin/renderer/HtmlToHtml");
const Setting_1 = require("@joplin/lib/models/Setting");
const usePrevious_1 = require("../../hooks/usePrevious");
const attachedResources_1 = require("@joplin/lib/utils/attachedResources");
const renderer_1 = require("@joplin/renderer");
const Note_1 = require("@joplin/lib/models/Note");
const ResourceFetcher_1 = require("@joplin/lib/services/ResourceFetcher");
const focusHandler_1 = require("@joplin/lib/utils/focusHandler");
const Logger_1 = require("@joplin/utils/Logger");
const eventManager_1 = require("@joplin/lib/eventManager");
const DecryptionWorker_1 = require("@joplin/lib/services/DecryptionWorker");
const useQueuedAsyncEffect_1 = require("@joplin/lib/hooks/useQueuedAsyncEffect");
const logger = Logger_1.default.create('useFormNote');
function installResourceChangeHandler(onResourceChangeHandler) {
    ResourceFetcher_1.default.instance().on('downloadComplete', onResourceChangeHandler);
    ResourceFetcher_1.default.instance().on('downloadStarted', onResourceChangeHandler);
    DecryptionWorker_1.default.instance().on('resourceDecrypted', onResourceChangeHandler);
    eventManager_1.default.on(eventManager_1.EventName.ResourceChange, onResourceChangeHandler);
}
function uninstallResourceChangeHandler(onResourceChangeHandler) {
    ResourceFetcher_1.default.instance().off('downloadComplete', onResourceChangeHandler);
    ResourceFetcher_1.default.instance().off('downloadStarted', onResourceChangeHandler);
    DecryptionWorker_1.default.instance().off('resourceDecrypted', onResourceChangeHandler);
    eventManager_1.default.off(eventManager_1.EventName.ResourceChange, onResourceChangeHandler);
}
function resourceInfosChanged(a, b) {
    if (Object.keys(a).length !== Object.keys(b).length)
        return true;
    for (const id in a) {
        const r1 = a[id];
        const r2 = b[id];
        if (!r2)
            return true;
        if (r1.item.updated_time !== r2.item.updated_time)
            return true;
        if (r1.item.encryption_applied !== r2.item.encryption_applied)
            return true;
        if (r1.item.is_shared !== r2.item.is_shared)
            return true;
        if (r1.localState.fetch_status !== r2.localState.fetch_status)
            return true;
    }
    return false;
}
const useRefreshFormNoteOnChange = (formNoteRef, editorId, noteId, initNoteState, builtInEditorVisible) => {
    // Increasing the value of this counter cancels any ongoing note refreshes and starts
    // a new refresh.
    const [formNoteRefreshScheduled, setFormNoteRefreshScheduled] = (0, react_1.useState)(0);
    const prevBuiltInEditorVisible = (0, usePrevious_1.default)(builtInEditorVisible);
    (0, useQueuedAsyncEffect_1.default)(async (event) => {
        if (formNoteRefreshScheduled <= 0)
            return;
        if (formNoteRef.current.hasChanged) {
            logger.info('Form note changed between scheduling a refresh and the refresh itself. Cancelling the refresh.');
            return;
        }
        logger.info('Sync has finished and note has never been changed - reloading it');
        const loadNote = async () => {
            const n = await Note_1.default.load(noteId);
            if (event.cancelled || formNoteRef.current.hasChanged)
                return;
            // Normally should not happened because if the note has been deleted via sync
            // it would not have been loaded in the editor (due to note selection changing
            // on delete)
            if (!n) {
                logger.warn('Trying to reload note that has been deleted:', noteId);
                return;
            }
            await initNoteState(n, false);
            if (event.cancelled)
                return;
            setFormNoteRefreshScheduled(0);
        };
        await loadNote();
    }, [formNoteRefreshScheduled, noteId, editorId, initNoteState]);
    const refreshFormNote = (0, react_1.useCallback)(() => {
        // Increase the counter to cancel any ongoing refresh attempts
        // and start a new one.
        setFormNoteRefreshScheduled(formNoteRefreshScheduled + 1);
    }, [formNoteRefreshScheduled]);
    // When switching from the plugin editor to the built-in editor, we refresh the note since the
    // plugin may have modified it via the data API.
    (0, react_1.useEffect)(() => {
        if (prevBuiltInEditorVisible !== builtInEditorVisible && builtInEditorVisible) {
            refreshFormNote();
        }
    }, [builtInEditorVisible, prevBuiltInEditorVisible, refreshFormNote]);
    (0, react_1.useEffect)(() => {
        if (!noteId)
            return () => { };
        let cancelled = false;
        const listener = ({ itemId, changeId }) => {
            // If this change came from the current editor, it should already be
            // handled by calls to `setFormNote`. If events from the current editor
            // aren't ignored, most user-activated note changes (e.g. a keypress)
            // cause the note to refresh. (Undesired refreshes can cause the cursor to jump).
            const isExternalChange = !(changeId !== null && changeId !== void 0 ? changeId : 'unknown').endsWith(editorId);
            if (itemId === noteId && !cancelled && isExternalChange) {
                if (formNoteRef.current.hasChanged)
                    return;
                refreshFormNote();
            }
        };
        eventManager_1.default.on(eventManager_1.EventName.ItemChange, listener);
        return () => {
            eventManager_1.default.off(eventManager_1.EventName.ItemChange, listener);
            cancelled = true;
        };
    }, [formNoteRef, noteId, editorId, refreshFormNote]);
};
function useFormNote(dependencies) {
    const { noteId, isProvisional, titleInputRef, editorRef, onBeforeLoad, onAfterLoad, builtInEditorVisible, editorId, } = dependencies;
    const [formNote, setFormNote] = (0, react_1.useState)((0, types_1.defaultFormNote)());
    const [isNewNote, setIsNewNote] = (0, react_1.useState)(false);
    const previousNoteId = (0, usePrevious_1.default)(formNote.id);
    const [resourceInfos, setResourceInfos] = (0, react_1.useState)({});
    const formNoteRef = (0, react_1.useRef)(formNote);
    formNoteRef.current = formNote;
    const initNoteState = (0, react_1.useCallback)(async (n, isNewNote) => {
        var _a;
        let originalCss = '';
        if (n.markup_language === renderer_1.MarkupToHtml.MARKUP_LANGUAGE_HTML) {
            const splitted = (0, HtmlToHtml_1.splitHtml)(n.body);
            originalCss = splitted.css;
        }
        const newFormNote = {
            id: n.id,
            title: n.title,
            body: n.body,
            is_todo: n.is_todo,
            parent_id: n.parent_id,
            deleted_time: n.deleted_time,
            is_conflict: n.is_conflict,
            bodyWillChangeId: 0,
            bodyChangeId: 0,
            markup_language: n.markup_language,
            saveActionQueue: new AsyncActionQueue_1.default(300),
            originalCss: originalCss,
            hasChanged: false,
            user_updated_time: n.user_updated_time,
            encryption_applied: n.encryption_applied,
        };
        logger.debug('Initializing note state');
        // Note that for performance reason,the call to setResourceInfos should
        // be first because it loads the resource infos in an async way. If we
        // swap them, the formNote will be updated first and rendered, then the
        // the resources will load, and the note will be re-rendered.
        const resources = await (0, attachedResources_1.default)(n.body);
        // If the user changes the note while resources are loading, this can lead to
        // a note being incorrectly marked as "unchanged".
        if (!isNewNote && ((_a = formNoteRef.current) === null || _a === void 0 ? void 0 : _a.hasChanged)) {
            logger.info('Cancelled note refresh -- form note changed while loading attached resources.');
            return null;
        }
        setResourceInfos(resources);
        setFormNote(newFormNote);
        formNoteRef.current = newFormNote;
        logger.debug('Resource info and form note set.');
        await (0, resourceHandling_1.handleResourceDownloadMode)(n.body);
        return newFormNote;
    }, []);
    useRefreshFormNoteOnChange(formNoteRef, editorId, noteId, initNoteState, builtInEditorVisible);
    (0, react_1.useEffect)(() => {
        if (!noteId) {
            if (formNote.id)
                setFormNote((0, types_1.defaultFormNote)());
            return () => { };
        }
        if (formNote.id === noteId)
            return () => { };
        let cancelled = false;
        logger.debug('Loading existing note', noteId);
        function handleAutoFocus(noteIsTodo) {
            if (!isProvisional)
                return;
            const focusSettingName = noteIsTodo ? 'newTodoFocus' : 'newNoteFocus';
            requestAnimationFrame(() => {
                if (Setting_1.default.value(focusSettingName) === 'title') {
                    if (titleInputRef.current)
                        (0, focusHandler_1.focus)('useFormNote::handleAutoFocus', titleInputRef.current);
                }
                else {
                    if (editorRef.current)
                        editorRef.current.execCommand({ name: 'editor.focus' });
                }
            });
        }
        async function loadNote() {
            const n = await Note_1.default.load(noteId);
            if (cancelled)
                return;
            if (!n)
                throw new Error(`Cannot find note with ID: ${noteId}`);
            logger.debug('Loaded note:', n);
            await onBeforeLoad({ formNote });
            const newFormNote = await initNoteState(n, true);
            setIsNewNote(isProvisional);
            await onAfterLoad({ formNote: newFormNote });
            handleAutoFocus(!!n.is_todo);
        }
        void loadNote();
        return () => {
            cancelled = true;
        };
        // eslint-disable-next-line @seiyab/react-hooks/exhaustive-deps -- Old code before rule was applied
    }, [noteId, isProvisional, formNote]);
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    const onResourceChange = (0, react_1.useCallback)(async (event = null) => {
        const resourceIds = await Note_1.default.linkedResourceIds(formNote.body);
        if (!event || resourceIds.indexOf(event.id) >= 0) {
            (0, attachedResources_1.clearResourceCache)();
            const newResourceInfos = await (0, attachedResources_1.default)(formNote.body);
            setResourceInfos(newResourceInfos);
        }
    }, [formNote.body]);
    (0, react_1.useEffect)(() => {
        installResourceChangeHandler(onResourceChange);
        return () => {
            uninstallResourceChangeHandler(onResourceChange);
        };
    }, [onResourceChange]);
    (0, react_1.useEffect)(() => {
        if (previousNoteId !== formNote.id) {
            void onResourceChange();
        }
    }, [previousNoteId, formNote.id, onResourceChange]);
    (0, react_1.useEffect)(() => {
        let cancelled = false;
        async function runEffect() {
            const r = await (0, attachedResources_1.default)(formNote.body);
            if (cancelled)
                return;
            setResourceInfos((previous) => {
                return resourceInfosChanged(previous, r) ? r : previous;
            });
        }
        void runEffect();
        return () => {
            cancelled = true;
        };
    }, [formNote.body]);
    // Currently, useFormNote relies on formNoteRef being up-to-date immediately after the editor
    // changes, with no delay during which async code can run. Even a small delay (e.g. that introduced
    // by a setState -> useEffect) can lead to a race condition. See https://github.com/laurent22/joplin/issues/8960.
    const onSetFormNote = (0, react_1.useCallback)(newFormNote => {
        let newNote;
        if (typeof newFormNote === 'function') {
            newNote = newFormNote(formNoteRef.current);
        }
        else {
            newNote = newFormNote;
        }
        formNoteRef.current = newNote;
        setFormNote(newNote);
    }, [setFormNote]);
    return {
        isNewNote,
        formNote,
        setFormNote: onSetFormNote,
        resourceInfos,
    };
}
exports.default = useFormNote;
//# sourceMappingURL=useFormNote.js.map