"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Note_1 = require("@joplin/lib/models/Note");
const test_utils_1 = require("@joplin/lib/testing/test-utils");
const react_hooks_1 = require("@testing-library/react-hooks");
const useFormNote_1 = require("./useFormNote");
const shim_1 = require("@joplin/lib/shim");
const Resource_1 = require("@joplin/lib/models/Resource");
const path_1 = require("path");
const _1 = require(".");
const defaultFormNoteProps = {
    noteId: '',
    isProvisional: false,
    titleInputRef: null,
    editorRef: null,
    onBeforeLoad: () => { },
    onAfterLoad: () => { },
    editorId: 'editor',
    builtInEditorVisible: false,
};
describe('useFormNote', () => {
    beforeEach(async () => {
        await (0, test_utils_1.setupDatabaseAndSynchronizer)(1);
        await (0, test_utils_1.switchClient)(1);
    });
    it('should update note when decryption completes', async () => {
        const testNote = await Note_1.default.save({ title: 'Test Note!' });
        const makeFormNoteProps = () => {
            return Object.assign(Object.assign({}, defaultFormNoteProps), { noteId: testNote.id });
        };
        const formNote = (0, react_hooks_1.renderHook)(props => (0, useFormNote_1.default)(props), {
            initialProps: makeFormNoteProps(),
        });
        await formNote.waitFor(() => {
            // id is falsy until after the first load of the form note.
            expect(formNote.result.current.formNote.id).not.toBeFalsy();
        });
        expect(formNote.result.current.formNote).toMatchObject({
            encryption_applied: 0,
            title: testNote.title,
        });
        await (0, react_hooks_1.act)(async () => {
            await Note_1.default.save({
                id: testNote.id,
                encryption_cipher_text: 'cipher_text',
                encryption_applied: 1,
            });
        });
        // Changing encryption_applied should cause a re-render
        await (0, react_hooks_1.act)(async () => {
            await formNote.waitFor(() => {
                expect(formNote.result.current.formNote).toMatchObject({
                    encryption_applied: 1,
                });
            });
        });
        await (0, react_hooks_1.act)(async () => {
            await Note_1.default.save({
                id: testNote.id,
                encryption_applied: 0,
                title: 'Test Note!',
            });
        });
        // Ending decryption should also cause a re-render
        await formNote.waitFor(() => {
            expect(formNote.result.current.formNote).toMatchObject({
                encryption_applied: 0,
            });
            // A larger-than-default timeout is needed to prevent CI failures:
        }, { timeout: 15000 });
        formNote.unmount();
    });
    // Lacking is_conflict has previously caused UI issues. See https://github.com/laurent22/joplin/pull/10913
    // for details.
    it('should preserve value of is_conflict on save', async () => {
        const testNote = await Note_1.default.save({ title: 'Test Note!', is_conflict: 1 });
        const makeFormNoteProps = () => {
            return Object.assign(Object.assign({}, defaultFormNoteProps), { noteId: testNote.id });
        };
        const formNote = (0, react_hooks_1.renderHook)(props => (0, useFormNote_1.default)(props), {
            initialProps: makeFormNoteProps(),
        });
        await formNote.waitFor(() => {
            expect(formNote.result.current.formNote).toMatchObject({
                is_conflict: 1,
                title: testNote.title,
            });
        });
        // Should preserve is_conflict after save.
        expect(await (0, _1.formNoteToNote)(formNote.result.current.formNote)).toMatchObject({
            is_conflict: 1,
            deleted_time: 0,
            title: testNote.title,
        });
        formNote.unmount();
    });
    it('should reload the note when it is changed outside of the editor', async () => {
        const note = await Note_1.default.save({ title: 'Test Note!', body: '...' });
        const props = Object.assign(Object.assign({}, defaultFormNoteProps), { noteId: note.id });
        const formNote = (0, react_hooks_1.renderHook)(props => (0, useFormNote_1.default)(props), {
            initialProps: props,
        });
        await formNote.waitFor(() => {
            expect(formNote.result.current.formNote.title).toBe('Test Note!');
        });
        // Simulate the note being modified outside the editor
        await (0, react_hooks_1.act)(async () => {
            await Note_1.default.save({ id: note.id, title: 'Modified' });
        });
        await formNote.waitFor(() => {
            expect(formNote.result.current.formNote.title).toBe('Modified');
        });
        formNote.unmount();
    });
    test('should refresh resource infos when changed outside the editor', async () => {
        let note = await Note_1.default.save({});
        note = await shim_1.default.attachFileToNote(note, (0, path_1.join)(test_utils_1.supportDir, 'sample.txt'));
        const resourceIds = Note_1.default.linkedItemIds(note.body);
        const resource = await Resource_1.default.load(resourceIds[0]);
        const makeFormNoteProps = () => {
            return Object.assign(Object.assign({}, defaultFormNoteProps), { noteId: note.id });
        };
        const formNote = (0, react_hooks_1.renderHook)(props => (0, useFormNote_1.default)(props), {
            initialProps: makeFormNoteProps(),
        });
        await formNote.waitFor(() => {
            return Object.values(formNote.result.current.resourceInfos).length > 0;
        });
        const initialResourceInfos = formNote.result.current.resourceInfos;
        expect(initialResourceInfos).toMatchObject({
            [resource.id]: { item: { id: resource.id } },
        });
        await (0, react_hooks_1.act)(async () => {
            await Resource_1.default.save(Object.assign(Object.assign({}, resource), { filename: 'test.txt' }));
        });
        await formNote.waitFor(() => {
            const resourceInfo = formNote.result.current.resourceInfos[resource.id];
            expect(resourceInfo.item).toMatchObject({
                id: resource.id, filename: 'test.txt',
            });
        });
        formNote.unmount();
    });
});
//# sourceMappingURL=useFormNote.test.js.map