"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
const types_1 = require("./types");
const editorCommandDeclarations_1 = require("../editorCommandDeclarations");
const CommandService_1 = require("@joplin/lib/services/CommandService");
const time_1 = require("@joplin/lib/time");
const registry_1 = require("@joplin/lib/registry");
const getWindowCommandPriority_1 = require("./getWindowCommandPriority");
const commandsWithDependencies = [
    require('../commands/showLocalSearch'),
    require('../commands/focusElementNoteTitle'),
    require('../commands/focusElementNoteBody'),
    require('../commands/pasteAsText'),
];
function editorCommandRuntime(declaration, editorRef, onBodyChange) {
    return {
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        execute: async (_context, ...args) => {
            if (!editorRef.current) {
                registry_1.reg.logger().warn('Received command, but editor is gone', declaration.name);
                return;
            }
            if (!editorRef.current.execCommand) {
                registry_1.reg.logger().warn('Received command, but editor cannot execute commands', declaration.name);
                return;
            }
            if (declaration.name === 'insertDateTime') {
                return editorRef.current.execCommand({
                    name: 'insertText',
                    value: time_1.default.formatMsToLocal(new Date().getTime()),
                });
            }
            else if (declaration.name === 'scrollToHash') {
                return editorRef.current.scrollTo({
                    type: types_1.ScrollOptionTypes.Hash,
                    value: args[0],
                });
            }
            else if (declaration.name === 'editor.setText') {
                onBodyChange({ content: args[0], changeId: 0 });
            }
            else {
                return editorRef.current.execCommand({
                    name: declaration.name,
                    value: args[0],
                });
            }
        },
        // We disable the editor commands whenever a modal dialog is visible,
        // otherwise the user might type something in a dialog and accidentally
        // change something in the editor. However, we still enable them for
        // GotoAnything so that it's possible to type eg `textBold` and bold the
        // currently selected text.
        //
        // https://github.com/laurent22/joplin/issues/5707
        enabledCondition: (0, editorCommandDeclarations_1.enabledCondition)(declaration.name),
    };
}
function useWindowCommandHandler(dependencies) {
    const { setShowLocalSearch, noteSearchBarRef, editorRef, titleInputRef, onBodyChange, containerRef } = dependencies;
    (0, react_1.useEffect)(() => {
        const getRuntimePriority = () => (0, getWindowCommandPriority_1.default)(containerRef);
        const deregisterCallbacks = [];
        for (const declaration of editorCommandDeclarations_1.default) {
            const runtime = editorCommandRuntime(declaration, editorRef, onBodyChange);
            deregisterCallbacks.push(CommandService_1.default.instance().registerRuntime(declaration.name, Object.assign(Object.assign({}, runtime), { getPriority: getRuntimePriority }), true));
        }
        const dependencies = {
            editorRef,
            setShowLocalSearch,
            noteSearchBarRef,
            titleInputRef,
        };
        for (const command of commandsWithDependencies) {
            const runtime = command.runtime(dependencies);
            deregisterCallbacks.push(CommandService_1.default.instance().registerRuntime(command.declaration.name, Object.assign(Object.assign({}, runtime), { getPriority: getRuntimePriority }), true));
        }
        return () => {
            for (const runtime of deregisterCallbacks) {
                runtime.deregister();
            }
        };
    }, [editorRef, setShowLocalSearch, noteSearchBarRef, titleInputRef, onBodyChange, containerRef]);
}
exports.default = useWindowCommandHandler;
//# sourceMappingURL=useWindowCommandHandler.js.map