"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const theme_1 = require("@joplin/lib/theme");
const locale_1 = require("@joplin/lib/locale");
const NoteTextViewer_1 = require("./NoteTextViewer");
const HelpButton_1 = require("./HelpButton");
const BaseModel_1 = require("@joplin/lib/BaseModel");
const Revision_1 = require("@joplin/lib/models/Revision");
const RevisionService_1 = require("@joplin/lib/services/RevisionService");
const renderer_1 = require("@joplin/renderer");
const time_1 = require("@joplin/lib/time");
const bridge_1 = require("../services/bridge");
const urlUtils = require('@joplin/lib/urlUtils');
const ReactTooltip = require('react-tooltip');
const { connect } = require('react-redux');
const note_screen_shared_1 = require("@joplin/lib/components/shared/note-screen-shared");
const shim_1 = require("@joplin/lib/shim");
const react_1 = require("react");
const useQueuedAsyncEffect_1 = require("@joplin/lib/hooks/useQueuedAsyncEffect");
const useMarkupToHtml_1 = require("./hooks/useMarkupToHtml");
const useAsyncEffect_1 = require("@joplin/lib/hooks/useAsyncEffect");
const useNoteContent = (viewerRef, currentRevId, revisions, themeId, customCss, scrollbarSize) => {
    const [note, setNote] = (0, react_1.useState)(null);
    const markupToHtml = (0, useMarkupToHtml_1.default)({
        themeId,
        customCss,
        plugins: {},
        whiteBackgroundNoteRendering: false,
        scrollbarSize,
    });
    (0, useAsyncEffect_1.default)(async (event) => {
        if (!revisions.length || !currentRevId) {
            setNote(null);
        }
        else {
            const revIndex = BaseModel_1.default.modelIndexById(revisions, currentRevId);
            const note = await RevisionService_1.default.instance().revisionNote(revisions, revIndex);
            if (!note || event.cancelled)
                return;
            setNote(note);
        }
    }, [revisions, currentRevId, themeId, customCss, viewerRef]);
    (0, useQueuedAsyncEffect_1.default)(async () => {
        var _a, _b;
        const noteBody = (_a = note === null || note === void 0 ? void 0 : note.body) !== null && _a !== void 0 ? _a : (0, locale_1._)('This note has no history');
        const markupLanguage = (_b = note === null || note === void 0 ? void 0 : note.markup_language) !== null && _b !== void 0 ? _b : renderer_1.MarkupLanguage.Markdown;
        const result = await markupToHtml(markupLanguage, noteBody, {
            resources: await note_screen_shared_1.default.attachedResources(noteBody),
            whiteBackgroundNoteRendering: markupLanguage === renderer_1.MarkupLanguage.Html,
        });
        viewerRef.current.setHtml(result.html, {
            pluginAssets: result.pluginAssets,
        });
    }, [note, viewerRef]);
    return note;
};
const NoteRevisionViewerComponent = ({ themeId, noteId, onBack, customCss, scrollbarSize }) => {
    var _a;
    const helpButton_onClick = (0, react_1.useCallback)(() => { }, []);
    const viewerRef = (0, react_1.useRef)(null);
    const [revisions, setRevisions] = (0, react_1.useState)([]);
    const [currentRevId, setCurrentRevId] = (0, react_1.useState)('');
    const [restoring, setRestoring] = (0, react_1.useState)(false);
    const note = useNoteContent(viewerRef, currentRevId, revisions, themeId, customCss, scrollbarSize);
    const viewer_domReady = (0, react_1.useCallback)(async () => {
        // this.viewerRef_.current.openDevTools();
        const revisions = await Revision_1.default.allByType(BaseModel_1.default.TYPE_NOTE, noteId);
        setRevisions(revisions);
        setCurrentRevId(revisions.length ? revisions[revisions.length - 1].id : '');
    }, [noteId]);
    const importButton_onClick = (0, react_1.useCallback)(async () => {
        if (!note)
            return;
        setRestoring(true);
        await RevisionService_1.default.instance().importRevisionNote(note);
        setRestoring(false);
        await shim_1.default.showMessageBox(RevisionService_1.default.instance().restoreSuccessMessage(note), { type: shim_1.MessageBoxType.Info });
    }, [note]);
    const backButton_click = (0, react_1.useCallback)(() => {
        if (onBack)
            onBack();
    }, [onBack]);
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    const revisionList_onChange = (0, react_1.useCallback)((event) => {
        const value = event.target.value;
        if (!value) {
            if (onBack)
                onBack();
        }
        else {
            setCurrentRevId(value);
        }
    }, [onBack]);
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    const webview_ipcMessage = (0, react_1.useCallback)(async (event) => {
        // For the revision view, we only support a minimal subset of the IPC messages.
        // For example, we don't need interactive checkboxes or sync between viewer and editor view.
        // We try to get most links work though, except for internal (joplin://) links.
        const msg = event.channel ? event.channel : '';
        // const args = event.args;
        // if (msg !== 'percentScroll') console.info(`Got ipc-message: ${msg}`, args);
        try {
            if (msg.indexOf('joplin://') === 0) {
                throw new Error((0, locale_1._)('Unsupported link or message: %s', msg));
            }
            else if (urlUtils.urlProtocol(msg)) {
                await (0, bridge_1.default)().openExternal(msg);
            }
            else if (msg.indexOf('#') === 0) {
                // This is an internal anchor, which is handled by the WebView so skip this case
            }
            else {
                console.warn(`Unsupported message in revision view: ${msg}`);
            }
        }
        catch (error) {
            console.warn(error);
            (0, bridge_1.default)().showErrorMessageBox(error.message);
        }
    }, []);
    const theme = (0, theme_1.themeStyle)(themeId);
    const revisionListItems = [];
    const revs = revisions.slice().reverse();
    for (let i = 0; i < revs.length; i++) {
        const rev = revs[i];
        const stats = Revision_1.default.revisionPatchStatsText(rev);
        revisionListItems.push(React.createElement("option", { key: rev.id, value: rev.id }, `${time_1.default.formatMsToLocal(rev.item_updated_time)} (${stats})`));
    }
    const restoreButtonTitle = (0, locale_1._)('Restore');
    const helpMessage = (0, locale_1._)('Click "%s" to restore the note. It will be copied in the notebook named "%s". The current version of the note will not be replaced or modified.', restoreButtonTitle, RevisionService_1.default.instance().restoreFolderTitle());
    const titleInput = (React.createElement("div", { className: 'revision-viewer-title' },
        React.createElement("button", { onClick: backButton_click, style: Object.assign(Object.assign({}, theme.buttonStyle), { marginRight: 10, height: theme.inputStyle.height }) },
            React.createElement("i", { style: theme.buttonIconStyle, className: 'fa fa-chevron-left' }),
            (0, locale_1._)('Back')),
        React.createElement("input", { readOnly: true, type: "text", className: 'title', style: theme.inputStyle, value: (_a = note === null || note === void 0 ? void 0 : note.title) !== null && _a !== void 0 ? _a : '' }),
        React.createElement("select", { disabled: !revisions.length, value: currentRevId, className: 'revisions', style: theme.dropdownList, onChange: revisionList_onChange }, revisionListItems),
        React.createElement("button", { disabled: !revisions.length || restoring, onClick: importButton_onClick, className: 'restore', style: Object.assign(Object.assign({}, theme.buttonStyle), { marginLeft: 10, height: theme.inputStyle.height }) }, restoreButtonTitle),
        React.createElement(HelpButton_1.default, { tip: helpMessage, id: "noteRevisionHelpButton", onClick: helpButton_onClick })));
    const viewer = React.createElement(NoteTextViewer_1.default, { themeId: themeId, viewerStyle: { display: 'flex', flex: 1, borderLeft: 'none' }, ref: viewerRef, onDomReady: viewer_domReady, onIpcMessage: webview_ipcMessage });
    return (React.createElement("div", { className: 'revision-viewer-root' },
        titleInput,
        viewer,
        React.createElement(ReactTooltip, { place: "bottom", delayShow: 300, className: "help-tooltip" })));
};
const mapStateToProps = (state) => {
    return {
        themeId: state.settings.theme,
        scrollbarSize: state.settings['style.scrollbarSize'],
    };
};
const NoteRevisionViewer = connect(mapStateToProps)(NoteRevisionViewerComponent);
exports.default = NoteRevisionViewer;
//# sourceMappingURL=NoteRevisionViewer.js.map