"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const test_1 = require("./util/test");
const MainScreen_1 = require("./models/MainScreen");
const test_2 = require("@playwright/test");
const fs_extra_1 = require("fs-extra");
const path_1 = require("path");
const createStartupArgs_1 = require("./util/createStartupArgs");
const firstNonDevToolsWindow_1 = require("./util/firstNonDevToolsWindow");
const setFilePickerResponse_1 = require("./util/setFilePickerResponse");
const setMessageBoxResponse_1 = require("./util/setMessageBoxResponse");
const getImageSourceSize_1 = require("./util/getImageSourceSize");
test_1.test.describe('main', () => {
    (0, test_1.test)('app should launch', async ({ mainWindow }) => {
        // A window should open with the correct title
        (0, test_1.expect)(await mainWindow.title()).toMatch(/^Joplin/);
        const mainPage = await new MainScreen_1.default(mainWindow).setup();
        await mainPage.waitFor();
    });
    (0, test_1.test)('should be able to create and edit a new note', async ({ mainWindow }) => {
        const mainScreen = await new MainScreen_1.default(mainWindow).setup();
        const editor = await mainScreen.createNewNote('Test note');
        // Note list should contain the new note
        await (0, test_1.expect)(mainScreen.noteList.getNoteItemByTitle('Test note')).toBeVisible();
        // Focus the editor
        await editor.codeMirrorEditor.click();
        // Type some text
        await mainWindow.keyboard.type('# Test note!');
        await mainWindow.keyboard.press('Enter');
        await mainWindow.keyboard.press('Enter');
        await mainWindow.keyboard.type('New note content!');
        // Should render
        const viewerFrame = editor.getNoteViewerFrameLocator();
        await (0, test_1.expect)(viewerFrame.locator('h1')).toHaveText('Test note!');
    });
    (0, test_1.test)('mermaid and KaTeX should render', async ({ mainWindow }) => {
        const mainScreen = await new MainScreen_1.default(mainWindow).setup();
        const editor = await mainScreen.createNewNote('🚧 Test 🚧');
        const testCommitId = 'bf59b2';
        await editor.focusCodeMirrorEditor();
        const noteText = [
            '```mermaid',
            'gitGraph',
            '    commit id: "973193"',
            `    commit id: "${testCommitId}"`,
            '    branch dev',
            '    checkout dev',
            '    commit id: "ceea77"',
            '```',
            '',
            '',
            'KaTeX:',
            '$$ \\int_0^1 \\cos(2x + 3) $$',
            '',
            'Sum: $\\sum_{x=0}^{100} \\tan x$',
        ];
        let firstLine = true;
        for (const line of noteText) {
            if (line) {
                if (!firstLine) {
                    // Remove any auto-indentation, but avoid pressing shift-tab at
                    // the beginning of the editor.
                    await mainWindow.keyboard.press('Shift+Tab');
                }
                await mainWindow.keyboard.type(line);
            }
            await mainWindow.keyboard.press('Enter');
            firstLine = false;
        }
        // Should render mermaid
        const viewerFrame = editor.getNoteViewerFrameLocator();
        await (0, test_1.expect)(viewerFrame.locator('pre.mermaid text', { hasText: testCommitId })).toBeVisible();
        // Should render KaTeX (block)
        // toBeAttached: To be added to the DOM.
        await (0, test_1.expect)(viewerFrame.locator('.joplin-editable > .katex-display').first()).toBeAttached();
        await (0, test_1.expect)(viewerFrame.locator('.katex-display *', { hasText: 'cos' }).last()).toBeVisible();
        // Should render KaTeX (inline)
        await (0, test_1.expect)(viewerFrame.locator('.joplin-editable > .katex').first()).toBeAttached();
    });
    (0, test_1.test)('should correctly resize large images', async ({ electronApp, mainWindow }) => {
        const mainScreen = await new MainScreen_1.default(mainWindow).setup();
        await mainScreen.createNewNote('Image resize test (part 1)');
        const editor = mainScreen.noteEditor;
        await editor.focusCodeMirrorEditor();
        const filename = 'large-jpg-image-with-exif-rotation.jpg';
        await (0, setFilePickerResponse_1.default)(electronApp, [(0, path_1.join)(__dirname, 'resources', filename)]);
        // Should be possible to cancel attaching for large images
        await (0, setMessageBoxResponse_1.default)(electronApp, /^Cancel/i);
        await editor.attachFileButton.click();
        await (0, test_1.expect)(editor.codeMirrorEditor).toHaveText('', { useInnerText: true });
        // Clicking "No" should not resize
        await (0, setMessageBoxResponse_1.default)(electronApp, /^No/i);
        await editor.attachFileButton.click();
        const viewerFrame = editor.getNoteViewerFrameLocator();
        const renderedImage = viewerFrame.getByAltText(filename);
        const fullSize = await (0, getImageSourceSize_1.default)(renderedImage);
        // To make it easier to find the image (one image per note), we switch to a new, empty note.
        await mainScreen.createNewNote('Image resize test (part 2)');
        await editor.focusCodeMirrorEditor();
        // Clicking "Yes" should resize
        await (0, setMessageBoxResponse_1.default)(electronApp, /^Yes/i);
        await editor.attachFileButton.click();
        const resizedSize = await (0, getImageSourceSize_1.default)(renderedImage);
        (0, test_1.expect)(resizedSize[0]).toBeLessThan(fullSize[0]);
        (0, test_1.expect)(resizedSize[1]).toBeLessThan(fullSize[1]);
        // Should keep aspect ratio (regression test for #9597)
        (0, test_1.expect)(fullSize[0] / resizedSize[0]).toBeCloseTo(fullSize[1] / resizedSize[1]);
    });
    for (const target of ['', '_blank']) {
        (0, test_1.test)(`clicking on an external link with target=${JSON.stringify(target)} should try to launch a browser`, async ({ electronApp, mainWindow }) => {
            const mainScreen = await new MainScreen_1.default(mainWindow).setup();
            await mainScreen.waitFor();
            // Mock openExternal
            const nextExternalUrlPromise = electronApp.evaluate(({ shell }) => {
                return new Promise(resolve => {
                    const openExternal = async (url) => {
                        resolve(url);
                    };
                    shell.openExternal = openExternal;
                });
            });
            // Create a test link
            const testLinkTitle = 'This is a test link!';
            const linkHref = 'https://joplinapp.org/';
            await mainWindow.evaluate(({ testLinkTitle, linkHref, target }) => {
                const testLink = document.createElement('a');
                testLink.textContent = testLinkTitle;
                testLink.onclick = () => {
                    // We need to navigate by setting location.href -- clicking on a link
                    // directly within the main window (i.e. not in a PDF viewer) doesn't
                    // navigate.
                    location.href = linkHref;
                };
                testLink.href = '#';
                // Display on top of everything
                testLink.style.zIndex = '99999';
                testLink.style.position = 'fixed';
                testLink.style.top = '0';
                testLink.style.left = '0';
                if (target) {
                    testLink.target = target;
                }
                document.body.appendChild(testLink);
            }, { testLinkTitle, linkHref, target });
            const testLink = mainWindow.getByText(testLinkTitle);
            await (0, test_1.expect)(testLink).toBeVisible();
            await testLink.click({ noWaitAfter: true });
            (0, test_1.expect)(await nextExternalUrlPromise).toBe(linkHref);
        });
    }
    (0, test_1.test)('should start in safe mode if profile-dir/force-safe-mode-on-next-start exists', async ({ profileDirectory }) => {
        await (0, fs_extra_1.writeFile)((0, path_1.join)(profileDirectory, 'force-safe-mode-on-next-start'), 'true', 'utf8');
        // We need to write to the force-safe-mode file before opening the Electron app.
        // Open the app ourselves:
        const startupArgs = (0, createStartupArgs_1.default)(profileDirectory);
        const electronApp = await test_2._electron.launch({ args: startupArgs });
        const mainWindow = await (0, firstNonDevToolsWindow_1.default)(electronApp);
        const safeModeDisableLink = mainWindow.getByText('Disable safe mode and restart');
        await safeModeDisableLink.waitFor();
        await (0, test_1.expect)(safeModeDisableLink).toBeInViewport();
        await electronApp.close();
    });
});
//# sourceMappingURL=main.spec.js.map