"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const test_1 = require("./util/test");
const MainScreen_1 = require("./models/MainScreen");
const setFilePickerResponse_1 = require("./util/setFilePickerResponse");
const waitForNextOpenPath_1 = require("./util/waitForNextOpenPath");
const path_1 = require("path");
test_1.test.describe('richTextEditor', () => {
    (0, test_1.test)('HTML links should be preserved when editing a note', async ({ electronApp, mainWindow }) => {
        const mainScreen = await new MainScreen_1.default(mainWindow).setup();
        await mainScreen.createNewNote('Testing!');
        const editor = mainScreen.noteEditor;
        // Set the note's content
        await editor.focusCodeMirrorEditor();
        // Attach this file to the note (create a resource ID)
        await (0, setFilePickerResponse_1.default)(electronApp, [__filename]);
        await editor.attachFileButton.click();
        // Wait to render
        const viewerFrame = editor.getNoteViewerFrameLocator();
        await viewerFrame.locator('a[data-from-md]').waitFor();
        // Should have an attached resource
        const codeMirrorContent = await editor.codeMirrorEditor.innerText();
        const resourceUrlExpression = /\[.*\]\(:\/(\w+)\)/;
        (0, test_1.expect)(codeMirrorContent).toMatch(resourceUrlExpression);
        const resourceId = codeMirrorContent.match(resourceUrlExpression)[1];
        // Create a new note with just an HTML link
        await mainScreen.createNewNote('Another test');
        await editor.codeMirrorEditor.click();
        await mainWindow.keyboard.type(`<a href=":/${resourceId}">HTML Link</a>`);
        // Switch to the RTE
        await editor.toggleEditorsButton.click();
        await editor.richTextEditor.waitFor();
        // Edit the note to cause the original content to update
        await editor.getRichTextFrameLocator().locator('a').click();
        await mainWindow.keyboard.type('Test...');
        await editor.toggleEditorsButton.click();
        await editor.codeMirrorEditor.waitFor();
        // Note should still contain the resource ID and note title
        const finalCodeMirrorContent = await editor.codeMirrorEditor.innerText();
        (0, test_1.expect)(finalCodeMirrorContent).toContain(`:/${resourceId}`);
    });
    (0, test_1.test)('should watch resources for changes when opened with ctrl+click', async ({ electronApp, mainWindow }) => {
        const mainScreen = await new MainScreen_1.default(mainWindow).setup();
        await mainScreen.createNewNote('Testing!');
        const editor = mainScreen.noteEditor;
        // Set the note's content
        await editor.focusCodeMirrorEditor();
        // Attach this file to the note (create a resource ID)
        const pathToAttach = (0, path_1.join)(__dirname, 'resources', 'test-file.txt');
        await (0, setFilePickerResponse_1.default)(electronApp, [pathToAttach]);
        await editor.attachFileButton.click();
        // Switch to the RTE
        await editor.toggleEditorsButton.click();
        await editor.richTextEditor.waitFor();
        await editor.richTextEditor.click();
        // Click on the attached file URL
        const openPathResult = (0, waitForNextOpenPath_1.default)(electronApp);
        const targetLink = editor.getRichTextFrameLocator().getByRole('link', { name: (0, path_1.basename)(pathToAttach) });
        if (process.platform === 'darwin') {
            await targetLink.click({ modifiers: ['Meta'] });
        }
        else {
            await targetLink.click({ modifiers: ['Control'] });
        }
        // Should watch the file
        await mainWindow.getByText(/^The following attachments are being watched for changes/i).waitFor();
        (0, test_1.expect)(await openPathResult).toContain((0, path_1.basename)(pathToAttach));
    });
    (0, test_1.test)('pressing Tab should indent', async ({ mainWindow }) => {
        const mainScreen = await new MainScreen_1.default(mainWindow).setup();
        await mainScreen.createNewNote('Testing tabs!');
        const editor = mainScreen.noteEditor;
        await editor.toggleEditorsButton.click();
        await editor.richTextEditor.click();
        await mainWindow.keyboard.type('This is a');
        // Tab should add spaces
        await mainWindow.keyboard.press('Tab');
        await mainWindow.keyboard.type('test.');
        // Shift-tab should remove spaces
        await mainWindow.keyboard.press('Tab');
        await mainWindow.keyboard.press('Tab');
        await mainWindow.keyboard.press('Shift+Tab');
        await mainWindow.keyboard.type('Test!');
        // Escape then tab should move focus
        await mainWindow.keyboard.press('Escape');
        await (0, test_1.expect)(editor.richTextEditor).toBeFocused();
        await mainWindow.keyboard.press('Tab');
        await (0, test_1.expect)(editor.richTextEditor).not.toBeFocused();
        // After re-focusing the editor, Tab should indent again.
        await mainWindow.keyboard.press('Shift+Tab');
        await (0, test_1.expect)(editor.richTextEditor).toBeFocused();
        await mainWindow.keyboard.type(' Another:');
        await mainWindow.keyboard.press('Tab');
        await mainWindow.keyboard.type('!');
        // After switching back to the Markdown editor,
        await (0, test_1.expect)(editor.toggleEditorsButton).not.toBeDisabled();
        await editor.toggleEditorsButton.click();
        await (0, test_1.expect)(editor.codeMirrorEditor).toHaveText('This is a        test.        Test! Another:        !');
    });
    (0, test_1.test)('should be possible to navigate between the note title and rich text editor with enter/down/up keys', async ({ mainWindow }) => {
        const mainScreen = await new MainScreen_1.default(mainWindow).setup();
        await mainScreen.createNewNote('Testing keyboard navigation!');
        const editor = mainScreen.noteEditor;
        await editor.toggleEditorsButton.click();
        await editor.richTextEditor.waitFor();
        await editor.noteTitleInput.click();
        await (0, test_1.expect)(editor.noteTitleInput).toBeFocused();
        await mainWindow.keyboard.press('End');
        await mainWindow.keyboard.press('ArrowDown');
        await (0, test_1.expect)(editor.richTextEditor).toBeFocused();
        await mainWindow.keyboard.press('ArrowUp');
        await (0, test_1.expect)(editor.noteTitleInput).toBeFocused();
        await mainWindow.keyboard.press('Enter');
        await (0, test_1.expect)(editor.noteTitleInput).not.toBeFocused();
        await (0, test_1.expect)(editor.richTextEditor).toBeFocused();
    });
});
//# sourceMappingURL=richTextEditor.spec.js.map