"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const view_1 = require("@codemirror/view");
const getScrollFraction_1 = require("../getScrollFraction");
const codemirror_vim_1 = require("@replit/codemirror-vim");
const editorCommands_1 = require("../editorCommands/editorCommands");
const state_1 = require("@codemirror/state");
const Decorator_1 = require("./Decorator");
const insertLineAfter_1 = require("../editorCommands/insertLineAfter");
const CodeMirror5BuiltInOptions_1 = require("./CodeMirror5BuiltInOptions");
const { pregQuote } = require('@joplin/lib/string-utils-common');
const documentPositionFromPos = (doc, pos) => {
    const line = doc.lineAt(pos);
    return {
        // CM 5 uses 0-based line numbering
        line: line.number - 1,
        ch: pos - line.from,
    };
};
const posFromDocumentPosition = (doc, pos) => {
    const line = doc.line(pos.line + 1);
    return line.from + pos.ch;
};
class CodeMirror5Emulation extends codemirror_vim_1.CodeMirror {
    constructor(editor, logMessage) {
        super(editor);
        this.editor = editor;
        this.logMessage = logMessage;
        this._events = {};
        this._options = Object.create(null);
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        this._userExtensions = Object.create(null);
        // Used by some plugins to store state.
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        this.state = Object.create(null);
        this.Vim = codemirror_vim_1.Vim;
        // Passed as initial state to plugins
        this.Init = { toString: () => 'CodeMirror.Init' };
        this.commands = CodeMirror5Emulation.commands;
        const { decorator, extension: decoratorExtension } = Decorator_1.default.create(editor);
        this._decorator = decorator;
        this._decoratorExtension = decoratorExtension;
        this._builtInOptions = new CodeMirror5BuiltInOptions_1.default(editor);
        editor.dispatch({
            effects: state_1.StateEffect.appendConfig.of(this.makeCM6Extensions()),
        });
    }
    makeCM6Extensions() {
        const cm5 = this;
        const editor = this.editor;
        return [
            // Fires events
            view_1.EditorView.domEventHandlers({
                scroll: () => CodeMirror5Emulation.signal(this, 'scroll'),
                focus: () => CodeMirror5Emulation.signal(this, 'focus'),
                paste: event => CodeMirror5Emulation.signal(this, 'paste', event),
                blur: () => CodeMirror5Emulation.signal(this, 'blur'),
                mousedown: event => CodeMirror5Emulation.signal(this, 'mousedown', event),
            }),
            view_1.ViewPlugin.fromClass(class {
                update(update) {
                    try {
                        if (update.viewportChanged) {
                            CodeMirror5Emulation.signal(cm5, 'viewportChange', editor.viewport.from, editor.viewport.to);
                        }
                        if (update.docChanged) {
                            cm5.fireChangeEvents(update);
                            cm5.onChange(update);
                        }
                        if (update.selectionSet) {
                            cm5.onSelectionChange();
                        }
                        CodeMirror5Emulation.signal(cm5, 'update');
                        // Catch the error -- otherwise, CodeMirror will de-register the update listener.
                    }
                    catch (error) {
                        cm5.logMessage(`Error dispatching update: ${error}`);
                    }
                }
            }),
            // Decorations
            this._decoratorExtension,
            // Some plugins rely on a CodeMirror-measure element
            // to store temporary content.
            view_1.showPanel.of(() => {
                const dom = document.createElement('div');
                dom.classList.add('CodeMirror-measure');
                // Make invisible, but still measurable
                dom.style.visibility = 'hidden';
                dom.style.pointerEvents = 'none';
                dom.style.height = '0';
                dom.style.overflow = 'auto';
                return { dom };
            }),
            // Allows legacy CM5 CSS to apply to the editor:
            view_1.EditorView.editorAttributes.of({ class: 'CodeMirror' }),
        ];
    }
    isEventHandledBySuperclass(eventName) {
        return ['beforeSelectionChange'].includes(eventName);
    }
    on(eventName, callback) {
        var _a;
        var _b;
        if (this.isEventHandledBySuperclass(eventName)) {
            return super.on(eventName, callback);
        }
        (_a = (_b = this._events)[eventName]) !== null && _a !== void 0 ? _a : (_b[eventName] = []);
        this._events[eventName].push(callback);
    }
    off(eventName, callback) {
        if (!(eventName in this._events)) {
            return;
        }
        this._events[eventName] = this._events[eventName].filter(otherCallback => otherCallback !== callback);
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    static signal(target, eventName, ...args) {
        var _a;
        const listeners = (_a = target._events[eventName]) !== null && _a !== void 0 ? _a : [];
        for (const listener of listeners) {
            listener(target, ...args);
        }
        super.signal(target, eventName, ...args);
    }
    fireChangeEvents(update) {
        const changes = [];
        const origDoc = update.startState.doc;
        for (const transaction of update.transactions) {
            transaction.changes.iterChanges((fromA, toA, _fromB, _toB, inserted) => {
                changes.push({
                    from: documentPositionFromPos(origDoc, fromA),
                    to: documentPositionFromPos(origDoc, toA),
                    text: inserted.sliceString(0).split('\n'),
                    removed: origDoc.sliceString(fromA, toA).split('\n'),
                    transaction,
                });
            });
        }
        // Delay firing events -- event listeners may try to create transactions.
        // (this is done by the rich markdown plugin).
        setTimeout(() => {
            for (const change of changes) {
                CodeMirror5Emulation.signal(this, 'change', change);
                // If triggered by a user, also send the inputRead event
                if (change.transaction.isUserEvent('input')) {
                    CodeMirror5Emulation.signal(this, 'inputRead', change);
                }
            }
            CodeMirror5Emulation.signal(this, 'changes', changes);
        }, 0);
    }
    // codemirror-vim's adapter doesn't match the CM5 docs -- wrap it.
    getCursor(mode) {
        if (mode === 'from') {
            mode = 'start';
        }
        if (mode === 'to') {
            mode = 'end';
        }
        return super.getCursor(mode);
    }
    getSearchCursor(query, pos) {
        // The superclass CodeMirror adapter only supports regular expression
        // arguments.
        if (typeof query === 'string') {
            query = new RegExp(pregQuote(query));
        }
        return super.getSearchCursor(query, pos || { line: 0, ch: 0 });
    }
    lineAtHeight(height, _mode) {
        const lineInfo = this.editor.lineBlockAtHeight(height);
        // - 1: Convert to zero-based.
        const lineNumber = this.editor.state.doc.lineAt(lineInfo.to).number - 1;
        return lineNumber;
    }
    heightAtLine(lineNumber, mode) {
        // CodeMirror 5 uses 0-based line numbers. CM6 uses 1-based
        // line numbers.
        const doc = this.editor.state.doc;
        const lineInfo = doc.line(Math.min(lineNumber + 1, doc.lines));
        const lineBlock = this.editor.lineBlockAt(lineInfo.from);
        const height = lineBlock.top;
        if (mode === 'local') {
            const editorTop = this.editor.lineBlockAt(0).top;
            return height - editorTop;
        }
        else {
            return height;
        }
    }
    lineInfo(lineNumber) {
        const line = this.editor.state.doc.line(lineNumber + 1);
        const result = {
            line: lineNumber,
            // Note: In CM5, a line handle is not just a line number
            handle: lineNumber,
            text: line.text,
            // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
            gutterMarkers: [],
            textClass: ['cm-line', ...this._decorator.getLineClasses(lineNumber)],
            bgClass: '',
            wrapClass: '',
            widgets: this._decorator.getLineWidgets(lineNumber),
        };
        return result;
    }
    getStateAfter(_line) {
        // TODO: Should return parser state. Returning an empty object
        //       allows some plugins to run without crashing, however.
        return {};
    }
    getScrollPercent() {
        return (0, getScrollFraction_1.default)(this.editor);
    }
    // CodeMirror-Vim's scrollIntoView only supports pos as a DocumentPosition.
    scrollIntoView(pos, margin) {
        const isPosition = (arg) => {
            // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
            return arg.line !== undefined && arg.ch !== undefined;
        };
        if (isPosition(pos)) {
            return super.scrollIntoView(pos, margin);
        }
        else {
            return super.scrollIntoView(pos.from, margin);
        }
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    defineExtension(name, value) {
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        CodeMirror5Emulation.prototype[name] = value;
        this._userExtensions[name] = value;
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    defineOption(name, defaultValue, onUpdate) {
        this._options[name] = {
            value: defaultValue,
            onUpdate,
        };
        onUpdate(this, defaultValue, this.Init);
    }
    // Override codemirror-vim's setOption to allow user-defined options
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    setOption(name, value) {
        if (name in this._options) {
            const oldValue = this._options[name].value;
            this._options[name].value = value;
            this._options[name].onUpdate(this, value, oldValue);
        }
        else if (this._builtInOptions.supportsOption(name)) {
            this._builtInOptions.setOption(name, value);
        }
        else {
            super.setOption(name, value);
        }
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    getOption(name) {
        if (name in this._options) {
            return this._options[name].value;
        }
        else {
            return super.getOption(name);
        }
    }
    coordsChar(coords, mode) {
        // codemirror-vim's API only supports "div" mode. Thus, we convert
        // local to div:
        if (mode !== 'div') {
            const bbox = this.editor.contentDOM.getBoundingClientRect();
            coords = {
                left: coords.left - bbox.left,
                top: coords.top - bbox.top,
            };
        }
        return super.coordsChar(coords, 'div');
    }
    // codemirror-vim's API doesn't match the API docs here -- it expects addOverlay
    // to return a SearchQuery. As such, this override returns "any".
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    addOverlay(modeObject) {
        if ('query' in modeObject) {
            return super.addOverlay(modeObject);
        }
        return this._decorator.addOverlay(modeObject);
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    removeOverlay(overlay) {
        super.removeOverlay(overlay);
        this._decorator.removeOverlay(overlay);
    }
    addLineClass(lineNumber, where, className) {
        this._decorator.addLineClass(lineNumber, where, className);
    }
    removeLineClass(lineNumber, where, className) {
        this._decorator.removeLineClass(lineNumber, where, className);
    }
    addLineWidget(lineNumber, node, options) {
        return this._decorator.addLineWidget(lineNumber, node, options);
    }
    addWidget(pos, node) {
        if (node.parentElement) {
            node.remove();
        }
        const loc = posFromDocumentPosition(this.editor.state.doc, pos);
        const screenCoords = this.editor.coordsAtPos(loc);
        const bbox = this.editor.contentDOM.getBoundingClientRect();
        node.style.position = 'absolute';
        const left = screenCoords.left - bbox.left;
        node.style.left = `${left}px`;
        node.style.maxWidth = `${bbox.width - left}px`;
        node.style.top = `${screenCoords.top + this.editor.scrollDOM.scrollTop}px`;
        this.editor.scrollDOM.appendChild(node);
    }
    markText(from, to, options) {
        const doc = this.editor.state.doc;
        return this._decorator.markText(posFromDocumentPosition(doc, from), posFromDocumentPosition(doc, to), options);
    }
    getDoc() {
        // The emulation layer has several of the methods available on a CodeMirror 5 document.
        // For some plugins, `this` is sufficient.
        return this;
    }
    joplinCommandToCodeMirrorCommand(commandName) {
        const match = /^editor\.(.*)$/g.exec(commandName);
        if (!match || !(match[1] in CodeMirror5Emulation.commands)) {
            return null;
        }
        return match[1];
    }
    supportsJoplinCommand(commandName) {
        return this.joplinCommandToCodeMirrorCommand(commandName) in CodeMirror5Emulation.commands;
    }
    execJoplinCommand(joplinCommandName) {
        const commandName = this.joplinCommandToCodeMirrorCommand(joplinCommandName);
        if (commandName === null) {
            this.logMessage(`Unsupported Joplin command, ${joplinCommandName}`);
            return;
        }
        if (this.commandExists(commandName)) {
            return this.execCommand(commandName);
        }
    }
    commandExists(commandName) {
        return commandName in CodeMirror5Emulation.commands || typeof this._userExtensions[commandName] === 'function';
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    execCommand(name, ...args) {
        if (!this.commandExists(name)) {
            this.logMessage(`Unsupported CodeMirror command, ${name}`);
            return;
        }
        if (name in CodeMirror5Emulation.commands) {
            return CodeMirror5Emulation.commands[name](this);
        }
        else if (typeof this._userExtensions[name] === 'function') {
            return this._userExtensions[name].call(this, ...args);
        }
    }
}
CodeMirror5Emulation.commands = (() => {
    const commands = Object.assign(Object.assign({}, codemirror_vim_1.CodeMirror.commands), { vimInsertListElement: (codeMirror) => {
            (0, insertLineAfter_1.default)(codeMirror.cm6);
            codemirror_vim_1.Vim.handleKey(codeMirror, 'i', 'macro');
        } });
    for (const commandName in editorCommands_1.default) {
        const command = editorCommands_1.default[commandName];
        commands[commandName] = (codeMirror) => command(codeMirror.cm6);
    }
    // as any: Required to properly extend the base class -- without this,
    // the commands dictionary isn't known (by TypeScript) to have the same
    // properties as the commands dictionary in the parent class.
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    return commands;
})();
exports.default = CodeMirror5Emulation;
//# sourceMappingURL=CodeMirror5Emulation.js.map