"use strict";
// Handles adding decorations to the CodeMirror editor -- converts CodeMirror5-style calls
// to input accepted by CodeMirror 6
Object.defineProperty(exports, "__esModule", { value: true });
const view_1 = require("@codemirror/view");
const state_1 = require("@codemirror/state");
const language_1 = require("@codemirror/language");
const mapRangeConfig = {
    // Updates a range based on some change to the document
    map: (range, change) => {
        const from = change.mapPos(range.from);
        const to = change.mapPos(range.to);
        return Object.assign(Object.assign({}, range), { from: Math.min(from, to), to: Math.max(from, to) });
    },
};
const addLineDecorationEffect = state_1.StateEffect.define(mapRangeConfig);
const removeLineDecorationEffect = state_1.StateEffect.define(mapRangeConfig);
const addMarkDecorationEffect = state_1.StateEffect.define(mapRangeConfig);
const removeMarkDecorationEffect = state_1.StateEffect.define(mapRangeConfig);
const refreshOverlaysEffect = state_1.StateEffect.define();
const addLineWidgetEffect = state_1.StateEffect.define(mapRangeConfig);
const removeLineWidgetEffect = state_1.StateEffect.define();
class WidgetDecorationWrapper extends view_1.WidgetType {
    constructor(element, options) {
        super();
        this.element = element;
        this.options = options;
    }
    toDOM() {
        const container = document.createElement('div');
        this.element.remove();
        container.appendChild(this.element);
        if (this.options.className) {
            container.classList.add(this.options.className);
        }
        // Applies margins and related CSS:
        container.classList.add('cm-line');
        return container;
    }
}
class Decorator {
    constructor(editor) {
        this.editor = editor;
        this._effectDecorations = view_1.Decoration.none;
        this._nextLineWidgetId = 0;
        this._decorationCache = Object.create(null);
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        this._overlays = [];
        const decorator = this;
        this._extension = [
            // Overlay decorations -- recreate all decorations when the editor changes
            // (overlay decorations require parsing the document and may change output
            // when the editor/view changes.)
            view_1.ViewPlugin.fromClass(class {
                constructor(view) {
                    this.decorations = decorator.createOverlayDecorations(view);
                }
                update(update) {
                    const updated = false;
                    const doUpdate = () => {
                        if (updated)
                            return;
                        this.decorations = decorator.createOverlayDecorations(update.view);
                    };
                    if (update.viewportChanged || update.docChanged) {
                        doUpdate();
                    }
                    else {
                        for (const transaction of update.transactions) {
                            for (const effect of transaction.effects) {
                                if (effect.is(refreshOverlaysEffect)) {
                                    doUpdate();
                                    break;
                                }
                            }
                        }
                    }
                }
            }, {
                decorations: v => v.decorations,
            }),
            // Other decorations based on effects. See the decoration examples: https://codemirror.net/examples/decoration/
            // Note that EditorView.decorations.from is required for block widgets.
            state_1.StateField.define({
                create: () => view_1.Decoration.none,
                update: (_, viewUpdate) => decorator.updateEffectDecorations([viewUpdate]),
                provide: field => view_1.EditorView.decorations.from(field),
            }),
        ];
    }
    static create(editor) {
        const decorator = new Decorator(editor);
        return { decorator, extension: decorator._extension };
    }
    classNameToCssDecoration(className, isLineDecoration, id) {
        let decoration;
        if (className in this._decorationCache && id === undefined) {
            decoration = this._decorationCache[className];
        }
        else {
            const attributes = { class: className };
            if (isLineDecoration) {
                decoration = view_1.Decoration.line({ attributes, id });
            }
            else {
                decoration = view_1.Decoration.mark({ attributes, id });
            }
            this._decorationCache[className] = decoration;
        }
        return decoration;
    }
    updateEffectDecorations(transactions) {
        let decorations = this._effectDecorations;
        // Update decoration positions
        for (const transaction of transactions) {
            decorations = decorations.map(transaction.changes);
            // Add or remove decorations
            for (const effect of transaction.effects) {
                const isMarkDecoration = effect.is(addMarkDecorationEffect);
                const isLineDecoration = effect.is(addLineDecorationEffect);
                if (isMarkDecoration || isLineDecoration) {
                    const decoration = this.classNameToCssDecoration(effect.value.cssClass, isLineDecoration, effect.value.id);
                    const value = effect.value;
                    const from = effect.value.from;
                    // Line decorations are specified to have a size-zero range.
                    const to = isLineDecoration ? from : value.to;
                    decorations = decorations.update({
                        add: [decoration.range(from, to)],
                    });
                }
                else if (effect.is(removeLineDecorationEffect) || effect.is(removeMarkDecorationEffect)) {
                    const doc = transaction.state.doc;
                    const targetFrom = doc.lineAt(effect.value.from).from;
                    const targetTo = doc.lineAt(effect.value.to).to;
                    const targetId = effect.value.id;
                    const targetDecoration = this.classNameToCssDecoration(effect.value.cssClass, effect.is(removeLineDecorationEffect));
                    decorations = decorations.update({
                        // Returns true only for decorations that should be kept.
                        filter: (from, to, value) => {
                            if (targetId !== undefined) {
                                return value.spec.id !== effect.value.id;
                            }
                            const isInRange = from >= targetFrom && to <= targetTo;
                            return isInRange && value.eq(targetDecoration);
                        },
                    });
                }
                else if (effect.is(addLineWidgetEffect)) {
                    const options = effect.value.options;
                    const decoration = view_1.Decoration.widget({
                        widget: new WidgetDecorationWrapper(effect.value.element, options),
                        side: options.above ? -1 : 1,
                        block: true,
                    });
                    decorations = decorations.update({
                        add: [decoration.range(options.above ? effect.value.from : effect.value.to)],
                    });
                }
                else if (effect.is(removeLineWidgetEffect)) {
                    decorations = decorations.update({
                        // Returns true only for decorations that should be kept.
                        filter: (_from, _to, value) => {
                            var _a;
                            return ((_a = value.spec.widget) === null || _a === void 0 ? void 0 : _a.element) !== effect.value.element;
                        },
                    });
                }
            }
        }
        this._effectDecorations = decorations;
        return decorations;
    }
    createOverlayDecorations(view) {
        var _a, _b, _c;
        var _d;
        const makeDecoration = (tokenName, start, stop) => {
            const isLineDecoration = tokenName.startsWith('line-');
            // CM5 prefixes class names with cm-
            tokenName = `cm-${tokenName}`;
            const decoration = this.classNameToCssDecoration(tokenName, isLineDecoration);
            return decoration.range(start, stop);
        };
        const indentSize = view.state.facet(language_1.indentUnit).length;
        const newDecorations = [];
        for (const overlay of this._overlays) {
            const state = (_b = (_a = overlay.startState) === null || _a === void 0 ? void 0 : _a.call(overlay, indentSize)) !== null && _b !== void 0 ? _b : {};
            for (const { from, to } of view.visibleRanges) {
                const fromLine = view.state.doc.lineAt(from);
                const toLine = view.state.doc.lineAt(to);
                const fromLineNumber = fromLine.number;
                const toLineNumber = toLine.number;
                for (let i = fromLineNumber; i <= toLineNumber; i++) {
                    const line = view.state.doc.line(i);
                    const reader = new language_1.StringStream(line.text, view.state.tabSize, indentSize);
                    let lastPos = 0;
                    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
                    (_c = (_d = reader).baseToken) !== null && _c !== void 0 ? _c : (_d.baseToken = () => null);
                    while (!reader.eol()) {
                        const token = overlay.token(reader, state);
                        if (token) {
                            for (const className of token.split(/\s+/)) {
                                if (className.startsWith('line-')) {
                                    newDecorations.push(makeDecoration(className, line.from, line.from));
                                }
                                else {
                                    const from = lastPos + line.from;
                                    const to = reader.pos + line.from;
                                    newDecorations.push(makeDecoration(className, from, to));
                                }
                            }
                        }
                        if (reader.pos === lastPos) {
                            throw new Error('Mark decoration position did not increase -- overlays must advance with each call to .token()');
                        }
                        lastPos = reader.pos;
                    }
                }
            }
        }
        // Required by CodeMirror:
        // Should be sorted by from position, then by length.
        newDecorations.sort((a, b) => {
            if (a.from !== b.from) {
                return a.from - b.from;
            }
            return a.to - b.to;
        });
        // Per the documentation, new tokens should be added in
        // increasing order.
        const decorations = new state_1.RangeSetBuilder();
        for (const decoration of newDecorations) {
            decorations.add(decoration.from, decoration.to, decoration.value);
        }
        return decorations.finish();
    }
    addOverlay(modeObject) {
        this._overlays.push(modeObject);
        this.editor.dispatch({
            effects: [refreshOverlaysEffect.of(null)],
        });
        return {
            clear: () => this.removeOverlay(modeObject),
        };
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    removeOverlay(overlay) {
        this._overlays = this._overlays.filter(other => other !== overlay);
        this.editor.dispatch({
            effects: [refreshOverlaysEffect.of(null)],
        });
    }
    addRemoveLineClass(lineNumber, className, add) {
        // + 1: Convert from zero-indexed to one-indexed
        const line = this.editor.state.doc.line(lineNumber + 1);
        const effect = add ? addLineDecorationEffect : removeLineDecorationEffect;
        this.editor.dispatch({
            effects: effect.of({
                cssClass: className,
                from: line.from,
                to: line.to,
            }),
        });
    }
    addLineClass(lineNumber, _where, className) {
        this.addRemoveLineClass(lineNumber, className, true);
    }
    removeLineClass(lineNumber, _where, className) {
        this.addRemoveLineClass(lineNumber, className, false);
    }
    getLineClasses(lineNumber) {
        const line = this.editor.state.doc.line(lineNumber + 1);
        const lineClasses = [];
        this._effectDecorations.between(line.from, line.to, (from, to, decoration) => {
            var _a;
            if (from === line.from && to === line.to) {
                const className = (_a = decoration.spec) === null || _a === void 0 ? void 0 : _a.class;
                if (typeof className === 'string') {
                    lineClasses.push(className);
                }
            }
        });
        return lineClasses;
    }
    markText(from, to, options) {
        var _a;
        const effectOptions = {
            cssClass: (_a = options.className) !== null && _a !== void 0 ? _a : '',
            id: this._nextLineWidgetId++,
            from,
            to,
        };
        this.editor.dispatch({
            effects: addMarkDecorationEffect.of(effectOptions),
        });
        return {
            clear: () => {
                this.editor.dispatch({
                    effects: removeMarkDecorationEffect.of(effectOptions),
                });
            },
        };
    }
    createLineWidgetControl(node, options) {
        return {
            node,
            clear: () => {
                this.editor.dispatch({
                    effects: removeLineWidgetEffect.of({ element: node }),
                });
            },
            changed: () => {
                this.editor.requestMeasure();
            },
            className: options.className,
        };
    }
    getLineWidgets(lineNumber) {
        const line = this.editor.state.doc.line(lineNumber + 1);
        const lineWidgets = [];
        this._effectDecorations.between(line.from, line.to, (from, to, decoration) => {
            var _a;
            if (from >= line.from && from <= line.to && from === to) {
                const widget = (_a = decoration.spec) === null || _a === void 0 ? void 0 : _a.widget;
                if (widget && widget instanceof WidgetDecorationWrapper) {
                    lineWidgets.push(this.createLineWidgetControl(widget.element, widget.options));
                }
            }
        });
        return lineWidgets;
    }
    addLineWidget(lineNumber, node, options) {
        const line = this.editor.state.doc.line(lineNumber + 1);
        const lineWidgetOptions = {
            from: line.from,
            to: line.to,
            element: node,
            options,
        };
        this.editor.dispatch({
            effects: addLineWidgetEffect.of(lineWidgetOptions),
        });
        return this.createLineWidgetControl(node, options);
    }
}
exports.default = Decorator;
//# sourceMappingURL=Decorator.js.map