"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const view_1 = require("@codemirror/view");
const types_1 = require("../types");
const CodeMirror5Emulation_1 = require("./CodeMirror5Emulation/CodeMirror5Emulation");
const editorCommands_1 = require("./editorCommands/editorCommands");
const state_1 = require("@codemirror/state");
const markdownCommands_1 = require("./markdown/markdownCommands");
const search_1 = require("@codemirror/search");
const PluginLoader_1 = require("./pluginApi/PluginLoader");
const customEditorCompletion_1 = require("./pluginApi/customEditorCompletion");
const RegionSpec_1 = require("./utils/formatting/RegionSpec");
const toggleInlineSelectionFormat_1 = require("./utils/formatting/toggleInlineSelectionFormat");
const getSearchState_1 = require("./utils/getSearchState");
class CodeMirrorControl extends CodeMirror5Emulation_1.default {
    constructor(editor, _callbacks) {
        super(editor, _callbacks.onLogMessage);
        this._callbacks = _callbacks;
        this._userCommands = new Map();
        this.joplinExtensions = {
            // Some plugins want to enable autocompletion from *just* that plugin, without also
            // enabling autocompletion for text within code blocks (and other built-in completion
            // sources).
            // To support this, we need to provide extensions that wrap the built-in autocomplete.
            // See https://discuss.codemirror.net/t/autocompletion-merging-override-in-config/7853
            completionSource: (completionSource) => customEditorCompletion_1.editorCompletionSource.of(completionSource),
            enableLanguageDataAutocomplete: customEditorCompletion_1.enableLanguageDataAutocomplete,
        };
        this._pluginControl = new PluginLoader_1.default(this, _callbacks.onLogMessage);
        this.addExtension((0, customEditorCompletion_1.default)());
    }
    supportsCommand(name) {
        return name in editorCommands_1.default || this._userCommands.has(name) || super.commandExists(name);
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    execCommand(name, ...args) {
        let commandOutput;
        if (this._userCommands.has(name)) {
            commandOutput = this._userCommands.get(name)(...args);
        }
        else if (name in editorCommands_1.default) {
            commandOutput = editorCommands_1.default[name](this.editor, ...args);
        }
        else if (super.commandExists(name)) {
            commandOutput = super.execCommand(name, ...args);
        }
        else if (super.supportsJoplinCommand(name)) {
            commandOutput = super.execJoplinCommand(name);
        }
        if (name === types_1.EditorCommandType.Undo || name === types_1.EditorCommandType.Redo) {
            this._callbacks.onUndoRedo();
        }
        return commandOutput;
    }
    registerCommand(name, command) {
        this._userCommands.set(name, command);
    }
    undo() {
        this.execCommand(types_1.EditorCommandType.Undo);
        this._callbacks.onUndoRedo();
    }
    redo() {
        this.execCommand(types_1.EditorCommandType.Redo);
        this._callbacks.onUndoRedo();
    }
    select(anchor, head) {
        this.editor.dispatch(this.editor.state.update({
            selection: { anchor, head },
            scrollIntoView: true,
        }));
    }
    clearHistory() {
        this._callbacks.onClearHistory();
    }
    setScrollPercent(fraction) {
        const maxScroll = this.editor.scrollDOM.scrollHeight - this.editor.scrollDOM.clientHeight;
        this.editor.scrollDOM.scrollTop = fraction * maxScroll;
    }
    insertText(text, userEvent) {
        this.editor.dispatch(this.editor.state.replaceSelection(text), { userEvent });
    }
    wrapSelections(start, end) {
        const regionSpec = RegionSpec_1.RegionSpec.of({ template: { start, end } });
        this.editor.dispatch(this.editor.state.changeByRange(range => {
            const update = (0, toggleInlineSelectionFormat_1.default)(this.editor.state, regionSpec, range);
            if (!update.range.empty) {
                // Deselect the start and end characters (roughly preserve the original
                // selection).
                update.range = state_1.EditorSelection.range(update.range.from + start.length, update.range.to - end.length);
            }
            return update;
        }));
    }
    updateBody(newBody) {
        // TODO: doc.toString() can be slow for large documents.
        const currentBody = this.editor.state.doc.toString();
        if (newBody !== currentBody) {
            // For now, collapse the selection to a single cursor
            // to ensure that the selection stays within the document
            // (and thus avoids an exception).
            const mainCursorPosition = this.editor.state.selection.main.anchor;
            // The maximum cursor position needs to be calculated using the EditorState,
            // to correctly account for line endings.
            const maxCursorPosition = this.editor.state.toText(newBody).length;
            const newCursorPosition = Math.min(mainCursorPosition, maxCursorPosition);
            this.editor.dispatch(this.editor.state.update({
                changes: {
                    from: 0,
                    to: this.editor.state.doc.length,
                    insert: newBody,
                },
                selection: state_1.EditorSelection.cursor(newCursorPosition),
                scrollIntoView: true,
            }));
            return true;
        }
        return false;
    }
    updateLink(newLabel, newUrl) {
        (0, markdownCommands_1.updateLink)(newLabel, newUrl)(this.editor);
    }
    updateSettings(newSettings) {
        this._callbacks.onSettingsChange(newSettings);
    }
    getSearchState() {
        return (0, getSearchState_1.default)(this.editor.state);
    }
    setSearchState(newState) {
        if (newState.dialogVisible !== (0, search_1.searchPanelOpen)(this.editor.state)) {
            this.execCommand(newState.dialogVisible ? types_1.EditorCommandType.ShowSearch : types_1.EditorCommandType.HideSearch);
        }
        const query = new search_1.SearchQuery({
            search: newState.searchText,
            caseSensitive: newState.caseSensitive,
            regexp: newState.useRegex,
            replace: newState.replaceText,
        });
        this.editor.dispatch({
            effects: [
                search_1.setSearchQuery.of(query),
            ],
        });
    }
    scrollToText(text, scrollStrategy) {
        const doc = this.editor.state.doc;
        const index = doc.toString().indexOf(text);
        const textFound = index >= 0;
        if (textFound) {
            this.editor.dispatch({
                effects: view_1.EditorView.scrollIntoView(index, { y: scrollStrategy }),
            });
        }
        return textFound;
    }
    addStyles(...styles) {
        const compartment = new state_1.Compartment();
        this.editor.dispatch({
            effects: state_1.StateEffect.appendConfig.of(compartment.of(view_1.EditorView.theme(...styles))),
        });
        return {
            remove: () => {
                this.editor.dispatch({
                    effects: compartment.reconfigure([]),
                });
            },
        };
    }
    setContentScripts(plugins) {
        return this._pluginControl.setPlugins(plugins);
    }
    remove() {
        this._pluginControl.remove();
        this._callbacks.onRemove();
    }
    //
    // CodeMirror-specific methods
    //
    prependKeymap(bindings) {
        const compartment = new state_1.Compartment();
        this.editor.dispatch({
            effects: state_1.StateEffect.appendConfig.of([
                compartment.of(view_1.keymap.of(bindings)),
            ]),
        });
        return {
            remove: () => {
                this.editor.dispatch({
                    effects: compartment.reconfigure([]),
                });
            },
        };
    }
    addExtension(extension) {
        this.editor.dispatch({
            effects: state_1.StateEffect.appendConfig.of([extension]),
        });
    }
}
exports.default = CodeMirrorControl;
//# sourceMappingURL=CodeMirrorControl.js.map