"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const state_1 = require("@codemirror/state");
const language_1 = require("@codemirror/language");
const search_1 = require("@codemirror/search");
const highlight_1 = require("@lezer/highlight");
const view_1 = require("@codemirror/view");
const commands_1 = require("@codemirror/commands");
const view_2 = require("@codemirror/view");
const search_2 = require("@codemirror/search");
const commands_2 = require("@codemirror/commands");
const events_1 = require("../events");
const markdownCommands_1 = require("./markdown/markdownCommands");
const decoratorExtension_1 = require("./markdown/decoratorExtension");
const computeSelectionFormatting_1 = require("./markdown/computeSelectionFormatting");
const SelectionFormatting_1 = require("../SelectionFormatting");
const configFromSettings_1 = require("./configFromSettings");
const getScrollFraction_1 = require("./getScrollFraction");
const CodeMirrorControl_1 = require("./CodeMirrorControl");
const insertLineAfter_1 = require("./editorCommands/insertLineAfter");
const handlePasteEvent_1 = require("./utils/handlePasteEvent");
const biDirectionalTextExtension_1 = require("./utils/biDirectionalTextExtension");
const searchExtension_1 = require("./utils/searchExtension");
const isCursorAtBeginning_1 = require("./utils/isCursorAtBeginning");
const overwriteModeExtension_1 = require("./utils/overwriteModeExtension");
const handleLinkEditRequests_1 = require("./utils/handleLinkEditRequests");
view_1.EditorView.EDIT_CONTEXT = false;
const createEditor = (parentElement, props) => {
    const initialText = props.initialText;
    let settings = props.settings;
    props.onLogMessage('Initializing CodeMirror...');
    // Handles firing an event when the undo/redo stack changes
    let schedulePostUndoRedoDepthChangeId_ = null;
    let lastUndoDepth = 0;
    let lastRedoDepth = 0;
    const schedulePostUndoRedoDepthChange = (editor, doItNow = false) => {
        if (schedulePostUndoRedoDepthChangeId_ !== null) {
            if (doItNow) {
                clearTimeout(schedulePostUndoRedoDepthChangeId_);
            }
            else {
                return;
            }
        }
        schedulePostUndoRedoDepthChangeId_ = setTimeout(() => {
            schedulePostUndoRedoDepthChangeId_ = null;
            const newUndoDepth = (0, commands_1.undoDepth)(editor.state);
            const newRedoDepth = (0, commands_1.redoDepth)(editor.state);
            if (newUndoDepth !== lastUndoDepth || newRedoDepth !== lastRedoDepth) {
                props.onEvent({
                    kind: events_1.EditorEventType.UndoRedoDepthChange,
                    undoDepth: newUndoDepth,
                    redoDepth: newRedoDepth,
                });
                lastUndoDepth = newUndoDepth;
                lastRedoDepth = newRedoDepth;
            }
        }, doItNow ? 0 : 1000);
    };
    let currentDocText = props.initialText;
    const notifyDocChanged = (viewUpdate) => {
        if (viewUpdate.docChanged) {
            currentDocText = editor.state.doc.toString();
            props.onEvent({
                kind: events_1.EditorEventType.Change,
                value: currentDocText,
            });
            schedulePostUndoRedoDepthChange(editor);
        }
    };
    const globalSpellcheckEnabled = () => {
        return editor.contentDOM.spellcheck;
    };
    const notifySelectionChange = (viewUpdate) => {
        if (!viewUpdate.state.selection.eq(viewUpdate.startState.selection)) {
            const mainRange = viewUpdate.state.selection.main;
            const event = {
                kind: events_1.EditorEventType.SelectionRangeChange,
                anchor: mainRange.anchor,
                head: mainRange.head,
                from: mainRange.from,
                to: mainRange.to,
            };
            props.onEvent(event);
        }
    };
    const notifySelectionFormattingChange = (viewUpdate) => {
        const spellcheck = globalSpellcheckEnabled();
        // If we can't determine the previous formatting, post the update regardless
        if (!viewUpdate) {
            const formatting = (0, computeSelectionFormatting_1.default)(editor.state, spellcheck);
            props.onEvent({
                kind: events_1.EditorEventType.SelectionFormattingChange,
                formatting,
            });
        }
        else if (viewUpdate.docChanged || !viewUpdate.state.selection.eq(viewUpdate.startState.selection)) {
            // Only post the update if something changed
            const oldFormatting = (0, computeSelectionFormatting_1.default)(viewUpdate.startState, spellcheck);
            const newFormatting = (0, computeSelectionFormatting_1.default)(viewUpdate.state, spellcheck);
            if (!(0, SelectionFormatting_1.selectionFormattingEqual)(oldFormatting, newFormatting)) {
                props.onEvent({
                    kind: events_1.EditorEventType.SelectionFormattingChange,
                    formatting: newFormatting,
                });
            }
        }
    };
    // Returns a keyboard command that returns true (so accepts the keybind)
    // alwaysActive: true if this command should be registered even if ignoreModifiers is given.
    const keyCommand = (key, run, alwaysActive) => {
        return {
            key,
            run: editor => {
                if (settings.ignoreModifiers && !alwaysActive)
                    return false;
                return run(editor);
            },
        };
    };
    const historyCompartment = new state_1.Compartment();
    const dynamicConfig = new state_1.Compartment();
    // Give the default keymap low precedence so that it is overridden
    // by extensions with default precedence.
    const keymapConfig = state_1.Prec.low(view_2.keymap.of([
        // Custom mod-f binding: Toggle the external dialog implementation
        // (don't show/hide the Panel dialog).
        keyCommand('Mod-f', (editor) => {
            if ((0, search_1.searchPanelOpen)(editor.state)) {
                (0, search_1.closeSearchPanel)(editor);
            }
            else {
                (0, search_1.openSearchPanel)(editor);
            }
            return true;
        }),
        // Markdown formatting keyboard shortcuts
        keyCommand('Mod-b', markdownCommands_1.toggleBolded),
        keyCommand('Mod-i', markdownCommands_1.toggleItalicized),
        keyCommand('Mod-$', markdownCommands_1.toggleMath),
        keyCommand('Mod-`', markdownCommands_1.toggleCode),
        keyCommand('Mod-[', markdownCommands_1.decreaseIndent),
        keyCommand('Mod-]', markdownCommands_1.increaseIndent),
        keyCommand('Mod-k', handleLinkEditRequests_1.showLinkEditor),
        keyCommand('Tab', (view) => {
            if (settings.autocompleteMarkup) {
                return (0, markdownCommands_1.insertOrIncreaseIndent)(view);
            }
            // Use the default indent behavior (which doesn't adjust markup)
            return (0, commands_1.insertTab)(view);
        }, true),
        keyCommand('Shift-Tab', (view) => {
            // When at the beginning of the editor, allow shift-tab to act
            // normally.
            if ((0, isCursorAtBeginning_1.default)(view.state)) {
                return false;
            }
            return (0, markdownCommands_1.decreaseIndent)(view);
        }, true),
        keyCommand('Mod-Enter', (_) => {
            (0, insertLineAfter_1.default)(_);
            return true;
        }, true),
        keyCommand('ArrowUp', (view) => {
            if ((0, isCursorAtBeginning_1.default)(view.state) && props.onSelectPastBeginning) {
                props.onSelectPastBeginning();
                return true;
            }
            return false;
        }, true),
        ...commands_1.standardKeymap, ...commands_2.historyKeymap, ...search_2.searchKeymap,
    ]));
    const editor = new view_1.EditorView({
        state: state_1.EditorState.create({
            // See https://github.com/codemirror/basic-setup/blob/main/src/codemirror.ts
            // for a sample configuration.
            extensions: [
                keymapConfig,
                dynamicConfig.of((0, configFromSettings_1.default)(props.settings)),
                historyCompartment.of((0, commands_1.history)()),
                (0, searchExtension_1.default)(props.onEvent, props.settings),
                // Allows multiple selections and allows selecting a rectangle
                // with ctrl (as in CodeMirror 5)
                state_1.EditorState.allowMultipleSelections.of(true),
                (0, view_1.rectangularSelection)(),
                (0, view_1.drawSelection)(),
                (0, view_1.highlightSpecialChars)(),
                (0, language_1.indentOnInput)(),
                view_1.EditorView.domEventHandlers({
                    scroll: (_event, view) => {
                        props.onEvent({
                            kind: events_1.EditorEventType.Scroll,
                            fraction: (0, getScrollFraction_1.default)(view),
                        });
                    },
                    paste: (event, view) => {
                        if (props.onPasteFile) {
                            (0, handlePasteEvent_1.default)(event, view, props.onPasteFile);
                        }
                    },
                    dragover: (event, _view) => {
                        if (props.onPasteFile && event.dataTransfer.files.length) {
                            event.preventDefault();
                            event.dataTransfer.dropEffect = 'copy';
                            return true;
                        }
                        return false;
                    },
                    drop: (event, view) => {
                        if (props.onPasteFile) {
                            (0, handlePasteEvent_1.default)(event, view, props.onPasteFile);
                        }
                    },
                }),
                state_1.EditorState.tabSize.of(4),
                // Apply styles to entire lines (block-display decorations)
                decoratorExtension_1.default,
                (0, view_1.dropCursor)(),
                biDirectionalTextExtension_1.default,
                overwriteModeExtension_1.default,
                props.localisations ? state_1.EditorState.phrases.of(props.localisations) : [],
                // Adds additional CSS classes to tokens (the default CSS classes are
                // auto-generated and thus unstable).
                (0, language_1.syntaxHighlighting)(highlight_1.classHighlighter),
                view_1.EditorView.lineWrapping,
                view_1.EditorView.updateListener.of((viewUpdate) => {
                    notifyDocChanged(viewUpdate);
                    notifySelectionChange(viewUpdate);
                    notifySelectionFormattingChange(viewUpdate);
                }),
                (0, handleLinkEditRequests_1.default)(() => {
                    props.onEvent({
                        kind: events_1.EditorEventType.EditLink,
                    });
                }),
            ],
            doc: initialText,
        }),
        parent: parentElement,
    });
    const editorControls = new CodeMirrorControl_1.default(editor, {
        onClearHistory: () => {
            // Clear history by removing then re-add the history extension.
            // Just re-adding the history extension isn't enough.
            editor.dispatch({
                effects: historyCompartment.reconfigure([]),
            });
            editor.dispatch({
                effects: historyCompartment.reconfigure((0, commands_1.history)()),
            });
        },
        onSettingsChange: (newSettings) => {
            settings = newSettings;
            editor.dispatch({
                effects: dynamicConfig.reconfigure((0, configFromSettings_1.default)(newSettings)),
            });
        },
        onUndoRedo: () => {
            // This callback is triggered when undo/redo is called
            // directly. Show visual feedback immediately.
            schedulePostUndoRedoDepthChange(editor, true);
        },
        onLogMessage: props.onLogMessage,
        onRemove: () => {
            editor.destroy();
        },
    });
    return editorControls;
};
exports.default = createEditor;
//# sourceMappingURL=createEditor.js.map