"use strict";
/**
 * @jest-environment jsdom
 */
Object.defineProperty(exports, "__esModule", { value: true });
const createEditor_1 = require("./createEditor");
const Setting_1 = require("@joplin/lib/models/Setting");
const language_1 = require("@codemirror/language");
const loadLanguages_1 = require("./testUtil/loadLanguages");
const globals_1 = require("@jest/globals");
const createEditorSettings_1 = require("./testUtil/createEditorSettings");
(0, globals_1.describe)('createEditor', () => {
    beforeAll(() => {
        jest.useFakeTimers();
        for (const scriptContainer of document.querySelectorAll('#joplin-plugin-scripts-container')) {
            scriptContainer.remove();
        }
    });
    // This checks for a regression -- occasionally, when updating packages,
    // syntax highlighting in the CodeMirror editor stops working. This is usually
    // fixed by
    // 1. removing all `@codemirror/` and `@lezer/` dependencies from yarn.lock,
    // 2. upgrading all CodeMirror packages to the latest versions in package.json, and
    // 3. re-running `yarn install`.
    //
    // See https://github.com/laurent22/joplin/issues/7253
    (0, globals_1.it)('should give headings a different style', async () => {
        const headerLineText = '# Testing...';
        const initialText = `${headerLineText}\nThis is a test.`;
        const editorSettings = (0, createEditorSettings_1.default)(Setting_1.default.THEME_LIGHT);
        await (0, loadLanguages_1.default)();
        const editor = (0, createEditor_1.default)(document.body, {
            initialText,
            settings: editorSettings,
            onEvent: _event => { },
            onLogMessage: _message => { },
            onPasteFile: null,
        });
        // Force the generation of the syntax tree now.
        (0, language_1.forceParsing)(editor.editor);
        const headerLine = document.body.querySelector('.cm-headerLine');
        (0, globals_1.expect)(headerLine.textContent).toBe(headerLineText);
        (0, globals_1.expect)(getComputedStyle(headerLine).fontSize).toBe('1.6em');
        // CodeMirror nests the tag that styles the header within .cm-headerLine:
        //  <div class='cm-headerLine'><span class='someclass'>Testing...</span></div>
        const headerLineContent = document.body.querySelectorAll('.cm-headerLine > span');
        (0, globals_1.expect)(headerLineContent.length).toBeGreaterThanOrEqual(1);
        // Cleanup
        editor.remove();
    });
    (0, globals_1.it)('should support loading plugins', async () => {
        const initialText = '# Test\nThis is a test.';
        const editorSettings = (0, createEditorSettings_1.default)(Setting_1.default.THEME_LIGHT);
        const editor = (0, createEditor_1.default)(document.body, {
            initialText,
            settings: editorSettings,
            onEvent: _event => { },
            onLogMessage: _message => { },
            onPasteFile: null,
        });
        const getContentScriptJs = jest.fn(async () => {
            return `
				exports.default = context => {
					context.postMessage(context.pluginId);
					return {};
				};
			`;
        });
        const postMessageHandler = jest.fn();
        const testPlugin1 = {
            pluginId: 'a.plugin.id',
            contentScriptId: 'a.plugin.id.contentScript',
            loadCssAsset: async (_name) => '* { color: red; }',
            contentScriptJs: getContentScriptJs,
            postMessageHandler,
        };
        const testPlugin2 = {
            pluginId: 'another.plugin.id',
            contentScriptId: 'another.plugin.id.contentScript',
            loadCssAsset: async (_name) => '...',
            contentScriptJs: getContentScriptJs,
            postMessageHandler,
        };
        // Should be able to load a plugin
        await editor.setContentScripts([
            testPlugin1,
        ]);
        // Allow plugins to load
        await jest.runAllTimersAsync();
        // Because plugin loading is done by adding script elements to the document,
        // we test for the presence of these script elements, rather than waiting for
        // them to run.
        (0, globals_1.expect)(document.querySelectorAll('#joplin-plugin-scripts-container script')).toHaveLength(1);
        // Only one script should be present.
        const scriptContainer = document.querySelector('#joplin-plugin-scripts-container');
        (0, globals_1.expect)(scriptContainer.querySelectorAll('script')).toHaveLength(1);
        // Adding another plugin should add another script element
        await editor.setContentScripts([
            testPlugin2, testPlugin1,
        ]);
        await jest.runAllTimersAsync();
        // There should now be script elements for each plugin
        (0, globals_1.expect)(scriptContainer.querySelectorAll('script')).toHaveLength(2);
        // Removing the editor should remove the script container
        editor.remove();
        (0, globals_1.expect)(document.querySelectorAll('#joplin-plugin-scripts-container script')).toHaveLength(0);
    });
    (0, globals_1.it)('should support multiple content scripts from the same plugin', async () => {
        const initialText = '# Test\nThis is a test.';
        const editorSettings = (0, createEditorSettings_1.default)(Setting_1.default.THEME_LIGHT);
        const editor = (0, createEditor_1.default)(document.body, {
            initialText,
            settings: editorSettings,
            onEvent: _event => { },
            onLogMessage: _message => { },
            onPasteFile: null,
        });
        const getContentScriptJs = jest.fn(async () => {
            return `
				exports.default = context => {
					context.postMessage(context.pluginId);
					return {};
				};
			`;
        });
        const postMessageHandler = jest.fn();
        const pluginId = 'a.plugin.id';
        const testPlugin1 = {
            pluginId,
            contentScriptId: 'a.plugin.id.contentScript',
            loadCssAsset: async (_name) => '',
            contentScriptJs: getContentScriptJs,
            postMessageHandler,
        };
        const testPlugin2 = {
            pluginId,
            contentScriptId: 'another.plugin.id.contentScript',
            loadCssAsset: async (_name) => '',
            contentScriptJs: getContentScriptJs,
            postMessageHandler,
        };
        await editor.setContentScripts([
            testPlugin1, testPlugin2,
        ]);
        // Allows plugins to load
        await jest.runAllTimersAsync();
        // Should be one script container for each plugin
        (0, globals_1.expect)(document.querySelectorAll('#joplin-plugin-scripts-container script')).toHaveLength(2);
    });
});
//# sourceMappingURL=createEditor.test.js.map