"use strict";
// CodeMirror 6 commands that modify markdown formatting (e.g. toggleBold).
Object.defineProperty(exports, "__esModule", { value: true });
exports.updateLink = exports.decreaseIndent = exports.insertOrIncreaseIndent = exports.increaseIndent = exports.insertHorizontalRule = exports.toggleHeaderLevel = exports.toggleList = exports.toggleMath = exports.toggleCode = exports.toggleItalicized = exports.toggleBolded = void 0;
const types_1 = require("../../types");
const state_1 = require("@codemirror/state");
const language_1 = require("@codemirror/language");
const isInSyntaxNode_1 = require("../utils/isInSyntaxNode");
const toggleRegionFormatGlobally_1 = require("../utils/formatting/toggleRegionFormatGlobally");
const RegionSpec_1 = require("../utils/formatting/RegionSpec");
const toggleInlineFormatGlobally_1 = require("../utils/formatting/toggleInlineFormatGlobally");
const stripBlockquote_1 = require("./utils/stripBlockquote");
const isIndentationEquivalent_1 = require("../utils/formatting/isIndentationEquivalent");
const growSelectionToNode_1 = require("../utils/growSelectionToNode");
const tabsToSpaces_1 = require("../utils/formatting/tabsToSpaces");
const renumberSelectedLists_1 = require("./utils/renumberSelectedLists");
const toggleSelectedLinesStartWith_1 = require("../utils/formatting/toggleSelectedLinesStartWith");
const startingSpaceRegex = /^(\s*)/;
const toggleBolded = (view) => {
    const spec = RegionSpec_1.RegionSpec.of({ template: '**', nodeName: 'StrongEmphasis' });
    const changes = (0, toggleInlineFormatGlobally_1.default)(view.state, spec);
    view.dispatch(changes);
    return true;
};
exports.toggleBolded = toggleBolded;
const toggleItalicized = (view) => {
    let handledBoldItalicRegion = false;
    // Bold-italic regions' starting and ending patterns are similar to italicized regions.
    // Thus, we need additional logic to convert bold regions to bold-italic regions.
    view.dispatch(view.state.changeByRange((sel) => {
        const changes = [];
        // Only handle cursors (empty selections)
        if (sel.empty) {
            const doc = view.state.doc;
            const selLine = doc.lineAt(sel.from);
            const selStartLineIdx = sel.from - selLine.from;
            const selEndLineIdx = sel.to - selLine.from;
            const beforeSel = selLine.text.substring(0, selStartLineIdx);
            const afterSel = selLine.text.substring(selEndLineIdx);
            const isBolded = beforeSel.endsWith('**') && afterSel.startsWith('**');
            // If at the end of a bold-italic region, exit the region.
            if (afterSel.startsWith('***')) {
                sel = state_1.EditorSelection.cursor(sel.to + 3);
                handledBoldItalicRegion = true;
            }
            else if (isBolded) {
                // Create a bold-italic region.
                changes.push({
                    from: sel.from,
                    to: sel.to,
                    insert: '**',
                });
                // Move to the center of the bold-italic region (**|**** -> ***|***)
                sel = state_1.EditorSelection.cursor(sel.to + 1);
                handledBoldItalicRegion = true;
            }
        }
        return {
            changes,
            range: sel,
        };
    }));
    if (!handledBoldItalicRegion) {
        const changes = (0, toggleInlineFormatGlobally_1.default)(view.state, {
            nodeName: 'Emphasis',
            template: { start: '*', end: '*' },
            matcher: { start: /[_*]/g, end: /[_*]/g },
        });
        view.dispatch(changes);
    }
    return true;
};
exports.toggleItalicized = toggleItalicized;
// If the selected region is an empty inline code block, it will be converted to
// a block (fenced) code block.
const toggleCode = (view) => {
    const codeFenceRegex = /^```\w*\s*$/;
    const inlineRegionSpec = RegionSpec_1.RegionSpec.of({ template: '`', nodeName: 'InlineCode' });
    const blockRegionSpec = {
        nodeName: 'FencedCode',
        template: { start: '```', end: '```' },
        matcher: { start: codeFenceRegex, end: codeFenceRegex },
    };
    const changes = (0, toggleRegionFormatGlobally_1.default)(view.state, inlineRegionSpec, blockRegionSpec);
    view.dispatch(changes);
    return true;
};
exports.toggleCode = toggleCode;
const toggleMath = (view) => {
    const blockStartRegex = /^\$\$/;
    const blockEndRegex = /\$\$\s*$/;
    const inlineRegionSpec = RegionSpec_1.RegionSpec.of({ nodeName: 'InlineMath', template: '$' });
    const blockRegionSpec = RegionSpec_1.RegionSpec.of({
        nodeName: 'BlockMath',
        template: '$$',
        matcher: {
            start: blockStartRegex,
            end: blockEndRegex,
        },
    });
    const changes = (0, toggleRegionFormatGlobally_1.default)(view.state, inlineRegionSpec, blockRegionSpec);
    view.dispatch(changes);
    return true;
};
exports.toggleMath = toggleMath;
const toggleList = (listType) => {
    return (view) => {
        let state = view.state;
        let doc = state.doc;
        const orderedListTag = 'OrderedList';
        const unorderedListTag = 'BulletList';
        // RegExps for different list types. The regular expressions MUST
        // be mutually exclusive.
        // `(?!\[[ xX]+\])` means "not followed by [x] or [ ]".
        const bulletedRegex = /^\s*([-*])\s(?!\[[ xX]+\]\s)/;
        const checklistRegex = /^\s*[-*]\s\[[ xX]+\]\s/;
        const numberedRegex = /^\s*\d+\.\s/;
        const listRegexes = {
            [types_1.ListType.OrderedList]: numberedRegex,
            [types_1.ListType.CheckList]: checklistRegex,
            [types_1.ListType.UnorderedList]: bulletedRegex,
        };
        const getContainerType = (line) => {
            const lineContent = (0, stripBlockquote_1.default)(line);
            // Determine the container's type.
            const checklistMatch = lineContent.match(checklistRegex);
            const bulletListMatch = lineContent.match(bulletedRegex);
            const orderedListMatch = lineContent.match(numberedRegex);
            if (checklistMatch) {
                return types_1.ListType.CheckList;
            }
            else if (bulletListMatch) {
                return types_1.ListType.UnorderedList;
            }
            else if (orderedListMatch) {
                return types_1.ListType.OrderedList;
            }
            return null;
        };
        const changes = state.changeByRange((sel) => {
            const changes = [];
            let containerType = null;
            // Total number of characters added (deleted if negative)
            let charsAdded = 0;
            const originalSel = sel;
            let fromLine;
            let toLine;
            let firstLineIndentation;
            let firstLineInBlockQuote;
            let fromLineContent;
            const computeSelectionProps = () => {
                fromLine = doc.lineAt(sel.from);
                toLine = doc.lineAt(sel.to);
                fromLineContent = (0, stripBlockquote_1.default)(fromLine);
                firstLineIndentation = fromLineContent.match(startingSpaceRegex)[0];
                firstLineInBlockQuote = (fromLineContent !== fromLine.text);
                containerType = getContainerType(fromLine);
            };
            computeSelectionProps();
            const origFirstLineIndentation = firstLineIndentation;
            const origContainerType = containerType;
            // Grow `sel` to the smallest containing list, unless the
            // cursor is on an empty line, in which case, the user
            // probably wants to add a list item (and not select the entire
            // list).
            if (sel.empty && fromLine.text.trim() !== '') {
                sel = (0, growSelectionToNode_1.default)(state, sel, [orderedListTag, unorderedListTag]);
                computeSelectionProps();
            }
            // Reset the selection if it seems likely the user didn't want the selection
            // to be expanded
            const isIndentationDiff = !(0, isIndentationEquivalent_1.default)(state, firstLineIndentation, origFirstLineIndentation);
            if (isIndentationDiff) {
                const expandedRegionIndentation = firstLineIndentation;
                sel = originalSel;
                computeSelectionProps();
                // Use the indentation level of the expanded region if it's greater.
                // This makes sense in the case where unindented text is being converted to
                // the same type of list as its container. For example,
                //     1. Foobar
                // unindented text
                // that should be made a part of the above list.
                //
                // becoming
                //
                //     1. Foobar
                //     2. unindented text
                //     3. that should be made a part of the above list.
                const wasGreaterIndentation = ((0, tabsToSpaces_1.default)(state, expandedRegionIndentation).length
                    > (0, tabsToSpaces_1.default)(state, firstLineIndentation).length);
                if (wasGreaterIndentation) {
                    firstLineIndentation = expandedRegionIndentation;
                }
            }
            else if ((origContainerType !== containerType && (origContainerType !== null && origContainerType !== void 0 ? origContainerType : null) !== null)
                || containerType !== getContainerType(toLine)) {
                // If the container type changed, this could be an artifact of checklists/bulleted
                // lists sharing the same node type.
                // Find the closest range of the same type of list to the original selection
                let newFromLineNo = doc.lineAt(originalSel.from).number;
                let newToLineNo = doc.lineAt(originalSel.to).number;
                let lastFromLineNo;
                let lastToLineNo;
                while (newFromLineNo !== lastFromLineNo || newToLineNo !== lastToLineNo) {
                    lastFromLineNo = newFromLineNo;
                    lastToLineNo = newToLineNo;
                    if (lastFromLineNo - 1 >= 1) {
                        const testFromLine = doc.line(lastFromLineNo - 1);
                        if (getContainerType(testFromLine) === origContainerType) {
                            newFromLineNo--;
                        }
                    }
                    if (lastToLineNo + 1 <= doc.lines) {
                        const testToLine = doc.line(lastToLineNo + 1);
                        if (getContainerType(testToLine) === origContainerType) {
                            newToLineNo++;
                        }
                    }
                }
                sel = state_1.EditorSelection.range(doc.line(newFromLineNo).from, doc.line(newToLineNo).to);
                computeSelectionProps();
            }
            // Determine whether the expanded selection should be empty
            if (originalSel.empty && fromLine.number === toLine.number) {
                sel = state_1.EditorSelection.cursor(toLine.to);
            }
            // Select entire lines (if not just a cursor)
            if (!sel.empty) {
                sel = state_1.EditorSelection.range(fromLine.from, toLine.to);
            }
            // Number of the item in the list (e.g. 2 for the 2nd item in the list)
            let listItemCounter = 1;
            for (let lineNum = fromLine.number; lineNum <= toLine.number; lineNum++) {
                const line = doc.line(lineNum);
                const lineContent = (0, stripBlockquote_1.default)(line);
                const lineContentFrom = line.to - lineContent.length;
                const inBlockQuote = (lineContent !== line.text);
                const indentation = lineContent.match(startingSpaceRegex)[0];
                const wrongIndentation = !(0, isIndentationEquivalent_1.default)(state, indentation, firstLineIndentation);
                // If not the right list level,
                if (inBlockQuote !== firstLineInBlockQuote || wrongIndentation) {
                    // We'll be starting a new list
                    listItemCounter = 1;
                    continue;
                }
                // Don't add list numbers to otherwise empty lines (unless it's the first line)
                if (lineNum !== fromLine.number && line.text.trim().length === 0) {
                    // Do not reset the counter -- the markdown renderer doesn't!
                    continue;
                }
                const deleteFrom = lineContentFrom;
                let deleteTo = deleteFrom + indentation.length;
                // If we need to remove an existing list,
                const currentContainer = getContainerType(line);
                if (currentContainer !== null) {
                    const containerRegex = listRegexes[currentContainer];
                    const containerMatch = lineContent.match(containerRegex);
                    if (!containerMatch) {
                        throw new Error('Assertion failed: container regex does not match line content.');
                    }
                    deleteTo = lineContentFrom + containerMatch[0].length;
                }
                let replacementString;
                if (listType === containerType) {
                    // Delete the existing list if it's the same type as the current
                    replacementString = '';
                }
                else if (listType === types_1.ListType.OrderedList) {
                    replacementString = `${firstLineIndentation}${listItemCounter}. `;
                }
                else if (listType === types_1.ListType.CheckList) {
                    replacementString = `${firstLineIndentation}- [ ] `;
                }
                else {
                    replacementString = `${firstLineIndentation}- `;
                }
                changes.push({
                    from: deleteFrom,
                    to: deleteTo,
                    insert: replacementString,
                });
                charsAdded -= deleteTo - deleteFrom;
                charsAdded += replacementString.length;
                listItemCounter++;
            }
            // Don't change cursors to selections
            if (sel.empty) {
                // Position the cursor at the end of the last line modified
                sel = state_1.EditorSelection.cursor(toLine.to + charsAdded);
            }
            else {
                sel = state_1.EditorSelection.range(sel.from, sel.to + charsAdded);
            }
            return {
                changes,
                range: sel,
            };
        });
        view.dispatch(changes);
        state = view.state;
        doc = state.doc;
        // Renumber the list
        view.dispatch((0, renumberSelectedLists_1.default)(state));
        return true;
    };
};
exports.toggleList = toggleList;
const toggleHeaderLevel = (level) => {
    return (view) => {
        let headerStr = '';
        for (let i = 0; i < level; i++) {
            headerStr += '#';
        }
        const matchEmpty = true;
        // Remove header formatting for any other level
        let changes = (0, toggleSelectedLinesStartWith_1.default)(view.state, new RegExp(
        // Check all numbers of #s lower than [level]
        `${level - 1 >= 1 ? `(?:^[#]{1,${level - 1}}\\s)|` : ''
        // Check all number of #s higher than [level]
        }(?:^[#]{${level + 1},}\\s)`), '', matchEmpty);
        view.dispatch(changes);
        // Set to the proper header level
        changes = (0, toggleSelectedLinesStartWith_1.default)(view.state, 
        // We want exactly [level] '#' characters.
        new RegExp(`^[#]{${level}} `), `${headerStr} `, matchEmpty);
        view.dispatch(changes);
        return true;
    };
};
exports.toggleHeaderLevel = toggleHeaderLevel;
const insertHorizontalRule = (view) => {
    view.dispatch(view.state.changeByRange(selection => {
        const line = view.state.doc.lineAt(selection.to);
        const processedLineText = (0, stripBlockquote_1.default)(line);
        const inBlockQuote = processedLineText !== line.text;
        const needsNewLine = processedLineText !== '';
        let prefix = inBlockQuote && needsNewLine ? '> ' : '';
        if (needsNewLine) {
            prefix = `\n${prefix}`;
        }
        const insert = `${prefix}* * *`;
        return {
            range: state_1.EditorSelection.cursor(line.to + insert.length),
            changes: {
                from: line.to,
                insert,
            },
        };
    }));
    return true;
};
exports.insertHorizontalRule = insertHorizontalRule;
// Prepends the given editor's indentUnit to all lines of the current selection
// and re-numbers modified ordered lists (if any).
const increaseIndent = (view) => {
    const matchEmpty = true;
    const matchNothing = /$ ^/;
    const indentUnit = (0, language_1.indentString)(view.state, (0, language_1.getIndentUnit)(view.state));
    const changes = (0, toggleSelectedLinesStartWith_1.default)(view.state, 
    // Delete nothing
    matchNothing, 
    // ...and thus always add indentUnit.
    indentUnit, matchEmpty);
    view.dispatch(changes);
    // Fix any lists
    view.dispatch((0, renumberSelectedLists_1.default)(view.state));
    return true;
};
exports.increaseIndent = increaseIndent;
// Like `increaseIndent`, but may insert tabs, rather than
// indenting, in some instances.
const insertOrIncreaseIndent = (view) => {
    const selection = view.state.selection;
    const mainSelection = selection.main;
    if (selection.ranges.length !== 1 || !mainSelection.empty) {
        return (0, exports.increaseIndent)(view);
    }
    if ((0, isInSyntaxNode_1.default)(view.state, mainSelection, 'ListItem')) {
        return (0, exports.increaseIndent)(view);
    }
    const indentUnit = (0, language_1.indentString)(view.state, (0, language_1.getIndentUnit)(view.state));
    view.dispatch(view.state.changeByRange(selection => {
        return {
            // Move the selection to after the inserted text
            range: state_1.EditorSelection.cursor(selection.from + indentUnit.length),
            changes: {
                from: selection.from,
                insert: indentUnit,
            },
        };
    }));
    return true;
};
exports.insertOrIncreaseIndent = insertOrIncreaseIndent;
const decreaseIndent = (view) => {
    const matchEmpty = true;
    const changes = (0, toggleSelectedLinesStartWith_1.default)(view.state, 
    // Assume indentation is either a tab or in units
    // of n spaces.
    new RegExp(`^(?:[\\t]|[ ]{1,${(0, language_1.getIndentUnit)(view.state)}})`), 
    // Don't add new text
    '', matchEmpty);
    view.dispatch(changes);
    // Fix any lists
    view.dispatch((0, renumberSelectedLists_1.default)(view.state));
    return true;
};
exports.decreaseIndent = decreaseIndent;
const updateLink = (label, url) => {
    // Empty label? Just include the URL.
    const linkText = label === '' ? url : `[${label}](${url})`;
    return (editor) => {
        const transaction = editor.state.changeByRange((sel) => {
            const changes = [];
            // Search for a link that overlaps [sel]
            let linkFrom = null;
            let linkTo = null;
            (0, language_1.syntaxTree)(editor.state).iterate({
                from: sel.from, to: sel.to,
                enter: node => {
                    const haveFoundLink = (linkFrom !== null && linkTo !== null);
                    if (node.name === 'Link' || (node.name === 'URL' && !haveFoundLink)) {
                        linkFrom = node.from;
                        linkTo = node.to;
                    }
                },
            });
            linkFrom !== null && linkFrom !== void 0 ? linkFrom : (linkFrom = sel.from);
            linkTo !== null && linkTo !== void 0 ? linkTo : (linkTo = sel.to);
            changes.push({
                from: linkFrom, to: linkTo,
                insert: linkText,
            });
            return {
                changes,
                range: state_1.EditorSelection.range(linkFrom, linkFrom + linkText.length),
            };
        });
        editor.dispatch(transaction);
        return true;
    };
};
exports.updateLink = updateLink;
//# sourceMappingURL=markdownCommands.js.map