"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const state_1 = require("@codemirror/state");
const stripBlockquote_1 = require("./stripBlockquote");
const tabsToSpaces_1 = require("../../utils/formatting/tabsToSpaces");
const language_1 = require("@codemirror/language");
// Ensures that ordered lists within [sel] are numbered in ascending order.
const renumberSelectedLists = (state) => {
    const doc = state.doc;
    const listItemRegex = /^(\s*)(\d+)\.\s?/;
    // Re-numbers ordered lists and sublists with numbers on each line in [linesToHandle]
    const handleLines = (linesToHandle) => {
        var _a;
        const changes = [];
        if (linesToHandle.length === 0) {
            return changes;
        }
        let firstListNumber = Number((_a = listItemRegex.exec(linesToHandle[0].text)) === null || _a === void 0 ? void 0 : _a[2]);
        if (!isFinite(firstListNumber) || firstListNumber < 1) {
            firstListNumber = 1;
        }
        const listNumberStack = [];
        let currentGroupIndentation = '';
        let nextListNumber = firstListNumber;
        let prevLineNumber;
        for (const line of linesToHandle) {
            // Don't re-handle lines.
            if (line.number === prevLineNumber) {
                continue;
            }
            prevLineNumber = line.number;
            const filteredText = (0, stripBlockquote_1.default)(line);
            const match = filteredText.match(listItemRegex);
            // Skip lines that aren't the correct type (e.g. blank lines)
            if (!match) {
                continue;
            }
            const indentation = match[1];
            const indentationLen = (0, tabsToSpaces_1.default)(state, indentation).length;
            let targetIndentLen = (0, tabsToSpaces_1.default)(state, currentGroupIndentation).length;
            if (targetIndentLen < indentationLen) {
                listNumberStack.push({ nextListNumber, indentationLength: indentationLen });
                nextListNumber = 1;
            }
            else if (targetIndentLen > indentationLen) {
                nextListNumber = parseInt(match[2], 10);
                // Handle the case where we deindent multiple times. For example,
                // 1. test
                //    1. test
                //      1. test
                // 2. test
                while (targetIndentLen > indentationLen) {
                    const listNumberRecord = listNumberStack.pop();
                    if (!listNumberRecord) {
                        break;
                    }
                    else {
                        targetIndentLen = listNumberRecord.indentationLength;
                        nextListNumber = listNumberRecord.nextListNumber;
                    }
                }
            }
            if (targetIndentLen !== indentationLen) {
                currentGroupIndentation = indentation;
            }
            const from = line.to - filteredText.length;
            const to = from + match[0].length;
            const inserted = `${indentation}${nextListNumber}. `;
            nextListNumber++;
            changes.push({
                from,
                to,
                insert: inserted,
            });
        }
        return changes;
    };
    // Find all selected lists
    const selectedListRanges = [];
    for (const selection of state.selection.ranges) {
        const listLines = [];
        (0, language_1.syntaxTree)(state).iterate({
            from: selection.from,
            to: selection.to,
            enter: (nodeRef) => {
                if (nodeRef.name === 'ListItem') {
                    for (const node of nodeRef.node.parent.getChildren('ListItem')) {
                        const line = doc.lineAt(node.from);
                        const filteredText = (0, stripBlockquote_1.default)(line);
                        const match = filteredText.match(listItemRegex);
                        if (match) {
                            listLines.push(line);
                        }
                    }
                }
            },
        });
        listLines.sort((a, b) => a.number - b.number);
        if (listLines.length > 0) {
            const fromLine = listLines[0];
            const toLine = listLines[listLines.length - 1];
            selectedListRanges.push(state_1.EditorSelection.range(fromLine.from, toLine.to));
        }
    }
    const changes = [];
    if (selectedListRanges.length > 0) {
        // Use EditorSelection.create to merge overlapping lists
        const listsToHandle = state_1.EditorSelection.create(selectedListRanges).ranges;
        for (const listSelection of listsToHandle) {
            const lines = [];
            const startLine = doc.lineAt(listSelection.from);
            const endLine = doc.lineAt(listSelection.to);
            for (let i = startLine.number; i <= endLine.number; i++) {
                lines.push(doc.line(i));
            }
            changes.push(...handleLines(lines));
        }
    }
    return {
        changes,
    };
};
exports.default = renumberSelectedLists;
//# sourceMappingURL=renumberSelectedLists.js.map