"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const codeMirrorRequire_1 = require("./codeMirrorRequire");
let pluginScriptIdCounter = 0;
let pluginLoaderCounter = 0;
const contentScriptToId = (contentScript) => `${contentScript.pluginId}--${contentScript.contentScriptId}`;
class PluginLoader {
    constructor(editor, logMessage) {
        var _a, _b;
        var _c, _d;
        this.editor = editor;
        this.logMessage = logMessage;
        this.loadedContentScriptIds = [];
        this.pluginRemovalCallbacks = {};
        this.pluginScriptsContainer = document.createElement('div');
        this.pluginScriptsContainer.style.display = 'none';
        // For testing
        this.pluginScriptsContainer.id = 'joplin-plugin-scripts-container';
        document.body.appendChild(this.pluginScriptsContainer);
        // addPlugin works by creating <script> elements with the plugin's content. To pass
        // information to this <script>, we use global objects:
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        (_a = (_c = window).__pluginLoaderScriptLoadCallbacks) !== null && _a !== void 0 ? _a : (_c.__pluginLoaderScriptLoadCallbacks = Object.create(null));
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        (_b = (_d = window).__pluginLoaderRequireFunctions) !== null && _b !== void 0 ? _b : (_d.__pluginLoaderRequireFunctions = Object.create(null));
        this.pluginLoaderId = pluginLoaderCounter++;
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        window.__pluginLoaderRequireFunctions[this.pluginLoaderId] = codeMirrorRequire_1.default;
    }
    async setPlugins(contentScripts) {
        for (const contentScript of contentScripts) {
            const id = contentScriptToId(contentScript);
            if (!this.loadedContentScriptIds.includes(id)) {
                this.addPlugin(contentScript);
            }
        }
        // Remove old plugins
        const contentScriptIds = contentScripts.map(contentScriptToId);
        const removedIds = this.loadedContentScriptIds
            .filter(id => !contentScriptIds.includes(id));
        for (const id of removedIds) {
            if (id in this.pluginRemovalCallbacks) {
                this.pluginRemovalCallbacks[id]();
            }
        }
    }
    addPlugin(plugin) {
        const onRemoveCallbacks = [];
        this.logMessage(`Loading plugin ${plugin.pluginId}, content script ${plugin.contentScriptId}`);
        const addScript = (onLoad) => {
            const scriptElement = document.createElement('script');
            onRemoveCallbacks.push(() => {
                scriptElement.remove();
            });
            void (async () => {
                const scriptId = pluginScriptIdCounter++;
                const js = await plugin.contentScriptJs();
                // Stop if cancelled
                if (!this.loadedContentScriptIds.includes(contentScriptToId(plugin))) {
                    return;
                }
                scriptElement.appendChild(document.createTextNode(`
				(async () => {
					const exports = {};
					const module = { exports: exports };
					const require = window.__pluginLoaderRequireFunctions[${JSON.stringify(this.pluginLoaderId)}];
					const joplin = {
						require,
					};
		
					${js};
		
					window.__pluginLoaderScriptLoadCallbacks[${JSON.stringify(scriptId)}](module.exports);
				})();
				`));
                // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
                window.__pluginLoaderScriptLoadCallbacks[scriptId] = onLoad;
                this.pluginScriptsContainer.appendChild(scriptElement);
            })();
        };
        const addStyles = (cssStrings) => {
            // A container for style elements
            const styleContainer = document.createElement('div');
            onRemoveCallbacks.push(() => {
                styleContainer.remove();
            });
            for (const cssText of cssStrings) {
                const style = document.createElement('style');
                style.innerText = cssText;
                styleContainer.appendChild(style);
            }
            this.pluginScriptsContainer.appendChild(styleContainer);
        };
        this.pluginRemovalCallbacks[contentScriptToId(plugin)] = () => {
            for (const callback of onRemoveCallbacks) {
                callback();
            }
            this.loadedContentScriptIds = this.loadedContentScriptIds.filter(id => {
                return id !== contentScriptToId(plugin);
            });
        };
        addScript(async (exports) => {
            var _a, _b;
            if (!(exports === null || exports === void 0 ? void 0 : exports.default) || !(typeof exports.default === 'function')) {
                throw new Error('All plugins must have a function default export');
            }
            const context = {
                postMessage: plugin.postMessageHandler,
                pluginId: plugin.pluginId,
                contentScriptId: plugin.contentScriptId,
            };
            const loadedPlugin = (_a = exports.default(context)) !== null && _a !== void 0 ? _a : {};
            (_b = loadedPlugin.plugin) === null || _b === void 0 ? void 0 : _b.call(loadedPlugin, this.editor);
            if (loadedPlugin.codeMirrorOptions) {
                for (const key in loadedPlugin.codeMirrorOptions) {
                    this.editor.setOption(key, loadedPlugin.codeMirrorOptions[key]);
                }
            }
            if (loadedPlugin.assets) {
                const cssStrings = [];
                for (const asset of loadedPlugin.assets()) {
                    let assetText = asset.text;
                    let assetMime = asset.mime;
                    if (!asset.inline) {
                        if (!asset.name) {
                            throw new Error('Non-inline asset missing required property "name"');
                        }
                        if (assetMime !== 'text/css' && !asset.name.endsWith('.css')) {
                            throw new Error(`Non-css assets are not supported by the CodeMirror 6 editor. (Asset path: ${asset.name})`);
                        }
                        assetText = await plugin.loadCssAsset(asset.name);
                        assetMime = 'text/css';
                    }
                    if (assetMime !== 'text/css') {
                        throw new Error('Plugin assets must have property "mime" set to "text/css" or have a filename ending with ".css"');
                    }
                    cssStrings.push(assetText);
                }
                addStyles(cssStrings);
            }
        });
        this.loadedContentScriptIds.push(contentScriptToId(plugin));
    }
    remove() {
        this.pluginScriptsContainer.remove();
    }
}
exports.default = PluginLoader;
//# sourceMappingURL=PluginLoader.js.map