"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const state_1 = require("@codemirror/state");
const RegionSpec_1 = require("./RegionSpec");
const findInlineMatch_1 = require("./findInlineMatch");
describe('findInlineMatch', () => {
    jest.retryTimes(2);
    const boldSpec = RegionSpec_1.RegionSpec.of({
        template: '**',
    });
    it('matching a bolded region: should return the length of the match', () => {
        const doc = state_1.Text.of(['**test**']);
        const sel = state_1.EditorSelection.range(0, 5);
        // matchStart returns the length of the match
        expect((0, findInlineMatch_1.default)(doc, boldSpec, sel, findInlineMatch_1.MatchSide.Start)).toBe(2);
    });
    it('matching a bolded region: should match the end of a region, if next to the cursor', () => {
        const doc = state_1.Text.of(['**...** test.']);
        const sel = state_1.EditorSelection.range(5, 5);
        expect((0, findInlineMatch_1.default)(doc, boldSpec, sel, findInlineMatch_1.MatchSide.End)).toBe(2);
    });
    it('matching a bolded region: should return -1 if no match is found', () => {
        const doc = state_1.Text.of(['**...** test.']);
        const sel = state_1.EditorSelection.range(3, 3);
        expect((0, findInlineMatch_1.default)(doc, boldSpec, sel, findInlineMatch_1.MatchSide.Start)).toBe(-1);
    });
    it('should match a custom specification of italicized regions', () => {
        const spec = {
            template: { start: '*', end: '*' },
            matcher: { start: /[*_]/g, end: /[*_]/g },
        };
        const testString = 'This is a _test_';
        const testDoc = state_1.Text.of([testString]);
        const fullSel = state_1.EditorSelection.range('This is a '.length, testString.length);
        // should match the start of the region
        expect((0, findInlineMatch_1.default)(testDoc, spec, fullSel, findInlineMatch_1.MatchSide.Start)).toBe(1);
        // should match the end of the region
        expect((0, findInlineMatch_1.default)(testDoc, spec, fullSel, findInlineMatch_1.MatchSide.End)).toBe(1);
    });
    const listSpec = {
        template: { start: ' - ', end: '' },
        matcher: {
            start: /^\s*[-*]\s/g,
            end: /$/g,
        },
    };
    it('matching a custom list: should not match a list if not within the selection', () => {
        const doc = state_1.Text.of(['- Test...']);
        const sel = state_1.EditorSelection.range(1, 6);
        // Beginning of list not selected: no match
        expect((0, findInlineMatch_1.default)(doc, listSpec, sel, findInlineMatch_1.MatchSide.Start)).toBe(-1);
    });
    it('matching a custom list: should match start of selected, unindented list', () => {
        const doc = state_1.Text.of(['- Test...']);
        const sel = state_1.EditorSelection.range(0, 6);
        expect((0, findInlineMatch_1.default)(doc, listSpec, sel, findInlineMatch_1.MatchSide.Start)).toBe(2);
    });
    it('matching a custom list: should match start of indented list', () => {
        const doc = state_1.Text.of(['   - Test...']);
        const sel = state_1.EditorSelection.range(0, 6);
        expect((0, findInlineMatch_1.default)(doc, listSpec, sel, findInlineMatch_1.MatchSide.Start)).toBe(5);
    });
    it('matching a custom list: should match the end of an item in an indented list', () => {
        const doc = state_1.Text.of(['   - Test...']);
        const sel = state_1.EditorSelection.range(0, 6);
        // Zero-length, but found, selection
        expect((0, findInlineMatch_1.default)(doc, listSpec, sel, findInlineMatch_1.MatchSide.End)).toBe(0);
    });
});
//# sourceMappingURL=findInlineMatch.test.js.map