"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const state_1 = require("@codemirror/state");
const createTestEditor_1 = require("../testUtil/createTestEditor");
const overwriteModeExtension_1 = require("./overwriteModeExtension");
const typeText_1 = require("../testUtil/typeText");
const pressReleaseKey_1 = require("../testUtil/pressReleaseKey");
const createEditor = async (initialText, defaultEnabled = false) => {
    const editor = await (0, createTestEditor_1.default)(initialText, state_1.EditorSelection.cursor(0), [], [
        overwriteModeExtension_1.default,
    ]);
    if (defaultEnabled) {
        editor.dispatch({ effects: [overwriteModeExtension_1.toggleOverwrite.of(true)] });
    }
    return editor;
};
describe('overwriteModeExtension', () => {
    test('should be disabled by default', async () => {
        const editor = await createEditor('Test!');
        (0, typeText_1.default)(editor, 'This should be inserted. ');
        expect(editor.state.doc.toString()).toBe('This should be inserted. Test!');
    });
    test('should overwrite characters while typing', async () => {
        const editor = await createEditor('Test!', true);
        (0, pressReleaseKey_1.default)(editor, { key: 'A', code: 'KeyA' });
        (0, typeText_1.default)(editor, 'New');
        expect(editor.state.doc.toString()).toBe('Newt!');
    });
    test('should be toggled by pressing <insert>', async () => {
        const editor = await createEditor('Test!');
        (0, pressReleaseKey_1.default)(editor, { key: 'Insert', code: 'Insert' });
        expect((0, overwriteModeExtension_1.overwriteModeEnabled)(editor)).toBe(true);
        (0, typeText_1.default)(editor, 'Exam');
        expect(editor.state.doc.toString()).toBe('Exam!');
        (0, pressReleaseKey_1.default)(editor, { key: 'Insert', code: 'Insert' });
        (0, typeText_1.default)(editor, 'ple');
        expect(editor.state.doc.toString()).toBe('Example!');
    });
    test('should be disabled by pressing <escape>', async () => {
        const editor = await createEditor('Test!', true);
        expect((0, overwriteModeExtension_1.overwriteModeEnabled)(editor)).toBe(true);
        (0, pressReleaseKey_1.default)(editor, { key: 'Escape', code: 'Escape' });
        expect((0, overwriteModeExtension_1.overwriteModeEnabled)(editor)).toBe(false);
        // Escape should not re-enable the extension
        (0, pressReleaseKey_1.default)(editor, { key: 'Escape', code: 'Escape' });
        expect((0, overwriteModeExtension_1.overwriteModeEnabled)(editor)).toBe(false);
    });
    test('<insert> should not toggle overwrite mode if other keys are pressed', async () => {
        // On Linux, the Orca screen reader's default "do screen reader action" key is
        // <insert>. To avoid toggling insert mode when users perform screen reader actions,
        // pressing <insert> should only toggle insert mode in certain cases.
        const editor = await createEditor('Test');
        const insertKey = { code: 'Insert', key: 'Insert ' };
        editor.contentDOM.dispatchEvent(new KeyboardEvent('keydown', insertKey));
        expect((0, overwriteModeExtension_1.overwriteModeEnabled)(editor)).toBe(false);
        // Pressing & releasing the space key should prevent insert mode from being enabled
        (0, pressReleaseKey_1.default)(editor, { code: 'Space', key: 'Space' });
        editor.contentDOM.dispatchEvent(new KeyboardEvent('keyup', insertKey));
        expect((0, overwriteModeExtension_1.overwriteModeEnabled)(editor)).toBe(false);
    });
    test('should insert text if the cursor is at the end of a line', async () => {
        const editor = await createEditor('\nTest', true);
        (0, typeText_1.default)(editor, 'Test! This is a test! ');
        (0, typeText_1.default)(editor, 't');
        (0, typeText_1.default)(editor, 'e');
        (0, typeText_1.default)(editor, 's');
        (0, typeText_1.default)(editor, 't');
        expect(editor.state.doc.toString()).toBe('Test! This is a test! test\nTest');
    });
});
//# sourceMappingURL=overwriteModeExtension.test.js.map