"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.supportedMimeTypes = void 0;
const locale_1 = require("../../locale");
const Resource_1 = require("../../models/Resource");
const Setting_1 = require("../../models/Setting");
const shim_1 = require("../../shim");
const types_1 = require("../database/types");
const time_1 = require("@joplin/utils/time");
const Logger_1 = require("@joplin/utils/Logger");
const filterOcrText_1 = require("./utils/filterOcrText");
const TaskQueue_1 = require("../../TaskQueue");
const eventManager_1 = require("../../eventManager");
const logger = Logger_1.default.create('OcrService');
// From: https://github.com/naptha/tesseract.js/blob/master/docs/image-format.md
exports.supportedMimeTypes = [
    'application/pdf',
    'image/bmp',
    'image/jpeg',
    'image/jpg',
    'image/png',
    'image/webp',
    'image/x-portable-bitmap',
];
const resourceInfo = (resource) => {
    return `${resource.id} (type ${resource.mime})`;
};
class OcrService {
    constructor(driver) {
        this.isRunningInBackground_ = false;
        // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
        this.maintenanceTimer_ = null;
        this.pdfExtractDir_ = null;
        this.isProcessingResources_ = false;
        this.recognizeQueue_ = null;
        this.driver_ = driver;
        this.recognizeQueue_ = new TaskQueue_1.default('recognize', logger);
        this.recognizeQueue_.setConcurrency(5);
        this.recognizeQueue_.keepTaskResults = false;
    }
    async pdfExtractDir() {
        if (this.pdfExtractDir_ !== null)
            return this.pdfExtractDir_;
        const p = `${Setting_1.default.value('tempDir')}/ocr_pdf_extract`;
        await shim_1.default.fsDriver().mkdir(p);
        this.pdfExtractDir_ = p;
        return this.pdfExtractDir_;
    }
    get running() {
        return this.runInBackground;
    }
    async recognize(language, resource) {
        if (resource.encryption_applied)
            throw new Error(`Cannot OCR encrypted resource: ${resource.id}`);
        const resourceFilePath = Resource_1.default.fullPath(resource);
        if (resource.mime === 'application/pdf') {
            // OCR can be slow for large PDFs.
            // Skip it if the PDF already includes text.
            const pageTexts = await shim_1.default.pdfExtractEmbeddedText(resourceFilePath);
            const pagesWithText = pageTexts.filter(text => !!text.trim().length);
            if (pagesWithText.length > 0) {
                return {
                    text: pageTexts.join('\n'),
                };
            }
            const imageFilePaths = await shim_1.default.pdfToImages(resourceFilePath, await this.pdfExtractDir());
            const results = [];
            let pageIndex = 0;
            for (const imageFilePath of imageFilePaths) {
                logger.info(`Recognize: ${resourceInfo(resource)}: Processing PDF page ${pageIndex + 1} / ${imageFilePaths.length}...`);
                results.push(await this.driver_.recognize(language, imageFilePath));
                pageIndex++;
            }
            for (const imageFilePath of imageFilePaths) {
                await shim_1.default.fsDriver().remove(imageFilePath);
            }
            return {
                text: results.map(r => r.text).join('\n'),
            };
        }
        else {
            return this.driver_.recognize(language, resourceFilePath);
        }
    }
    async dispose() {
        await this.driver_.dispose();
    }
    async processResources() {
        if (this.isProcessingResources_)
            return;
        this.isProcessingResources_ = true;
        const totalResourcesToProcess = await Resource_1.default.needOcrCount(exports.supportedMimeTypes);
        const inProcessResourceIds = [];
        const skippedResourceIds = [];
        logger.info(`Found ${totalResourcesToProcess} resources to process...`);
        const makeQueueAction = (totalProcessed, language, resource) => {
            return async () => {
                logger.info(`Processing resource ${totalProcessed + 1} / ${totalResourcesToProcess}: ${resourceInfo(resource)}...`);
                const toSave = {
                    id: resource.id,
                };
                try {
                    const fetchStatus = await Resource_1.default.localState(resource.id);
                    if (fetchStatus.fetch_status === Resource_1.default.FETCH_STATUS_ERROR) {
                        throw new Error(`Cannot process resource ${resourceInfo(resource)} because it cannot be fetched from the server: ${fetchStatus.fetch_error}`);
                    }
                    if (fetchStatus.fetch_status !== Resource_1.default.FETCH_STATUS_DONE) {
                        skippedResourceIds.push(resource.id);
                        logger.info(`Skipping resource ${resourceInfo(resource)} because it has not been downloaded yet`);
                        return;
                    }
                    const result = await this.recognize(language, resource);
                    toSave.ocr_status = types_1.ResourceOcrStatus.Done;
                    toSave.ocr_text = (0, filterOcrText_1.default)(result.text);
                    toSave.ocr_details = Resource_1.default.serializeOcrDetails(result.lines);
                    toSave.ocr_error = '';
                }
                catch (error) {
                    const errorMessage = typeof error === 'string' ? error : error === null || error === void 0 ? void 0 : error.message;
                    logger.warn(`Could not process resource ${resourceInfo(resource)}`, error);
                    toSave.ocr_status = types_1.ResourceOcrStatus.Error;
                    toSave.ocr_text = '';
                    toSave.ocr_details = '';
                    toSave.ocr_error = errorMessage || 'Unknown error';
                }
                await Resource_1.default.save(toSave);
            };
        };
        try {
            const language = (0, locale_1.toIso639Alpha3)(Setting_1.default.value('locale'));
            let totalProcessed = 0;
            while (true) {
                const resources = await Resource_1.default.needOcr(exports.supportedMimeTypes, skippedResourceIds.concat(inProcessResourceIds), 100, {
                    fields: [
                        'id',
                        'mime',
                        'file_extension',
                        'encryption_applied',
                    ],
                });
                if (!resources.length)
                    break;
                for (const resource of resources) {
                    inProcessResourceIds.push(resource.id);
                    await this.recognizeQueue_.pushAsync(resource.id, makeQueueAction(totalProcessed++, language, resource));
                }
            }
            await this.recognizeQueue_.waitForAll();
            if (totalProcessed) {
                eventManager_1.default.emit(eventManager_1.EventName.OcrServiceResourcesProcessed);
            }
            logger.info(`${totalProcessed} resources have been processed.`);
        }
        finally {
            this.isProcessingResources_ = false;
        }
    }
    async maintenance() {
        await this.processResources();
    }
    async runInBackground() {
        if (this.isRunningInBackground_)
            return;
        this.isRunningInBackground_ = true;
        if (this.maintenanceTimer_)
            return;
        logger.info('Starting background service...');
        await this.maintenance();
        this.maintenanceTimer_ = shim_1.default.setInterval(async () => {
            await this.maintenance();
            this.maintenanceTimer_ = null;
        }, 5 * time_1.Minute);
    }
    async stopRunInBackground() {
        logger.info('Stopping background service...');
        if (this.maintenanceTimer_)
            shim_1.default.clearInterval(this.maintenanceTimer_);
        this.maintenanceTimer_ = null;
        this.isRunningInBackground_ = false;
        await this.recognizeQueue_.stop();
    }
}
exports.default = OcrService;
//# sourceMappingURL=OcrService.js.map