"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Setting_1 = require("@joplin/lib/models/Setting");
const eventManager_1 = require("@joplin/lib/eventManager");
const notesSortOrderUtils_1 = require("./notesSortOrderUtils");
const SUFFIX_FIELD = '$field';
const SUFFIX_REVERSE = '$reverse';
class PerFolderSortOrderService {
    static initialize() {
        this.loadPerFolderSortOrders();
        this.loadSharedSortOrder();
        eventManager_1.default.appStateOn('notesParentType', this.onFolderSelectionMayChange.bind(this, 'notesParentType'));
        eventManager_1.default.appStateOn('selectedFolderId', this.onFolderSelectionMayChange.bind(this, 'selectedFolderId'));
        eventManager_1.default.appStateOn('selectedSmartFilterId', this.onFolderSelectionMayChange.bind(this, 'selectedSmartFilterId'));
        this.previousFolderId = Setting_1.default.value('activeFolderId');
    }
    static isSet(folderId) {
        return folderId && this.perFolderSortOrders && this.perFolderSortOrders.hasOwnProperty(folderId + SUFFIX_FIELD);
    }
    static get(folderId) {
        if (folderId && this.perFolderSortOrders) {
            const field = this.perFolderSortOrders[folderId + SUFFIX_FIELD];
            const reverse = this.perFolderSortOrders[folderId + SUFFIX_REVERSE];
            if (field)
                return { field, reverse };
        }
        return undefined;
    }
    static set(folderId, own) {
        let targetId = folderId;
        const selectedId = this.getSelectedFolderId();
        if (!targetId) {
            targetId = selectedId; // default: selected folder
            if (!targetId)
                return;
        }
        const targetOwn = this.isSet(targetId);
        let newOwn;
        if (typeof own === 'undefined') {
            newOwn = !targetOwn; // default: toggling
        }
        else {
            newOwn = !!own;
            if (newOwn === targetOwn)
                return;
        }
        if (newOwn) {
            let field, reverse;
            if (!this.isSet(selectedId)) {
                field = Setting_1.default.value('notes.sortOrder.field');
                reverse = Setting_1.default.value('notes.sortOrder.reverse');
            }
            else {
                field = this.sharedSortOrder.field;
                if (Setting_1.default.value('notes.perFieldReversalEnabled')) {
                    reverse = this.sharedSortOrder[field];
                }
                else {
                    reverse = this.sharedSortOrder.reverse;
                }
            }
            PerFolderSortOrderService.setPerFolderSortOrder(targetId, field, reverse);
        }
        else {
            PerFolderSortOrderService.deletePerFolderSortOrder(targetId);
        }
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    static onFolderSelectionMayChange(cause, event) {
        if (cause !== 'notesParentType' && cause !== 'selectedFolderId' && cause !== 'selectedSmartFilterId') {
            return;
        }
        this.folderState[cause] = event.value;
        const selectedId = this.getSelectedFolderId();
        if (this.previousFolderId === selectedId)
            return;
        const field = Setting_1.default.value('notes.sortOrder.field');
        const reverse = Setting_1.default.value('notes.sortOrder.reverse');
        let previousFolderHasPerFolderSortOrder = false;
        if (this.previousFolderId !== null) {
            previousFolderHasPerFolderSortOrder = this.isSet(this.previousFolderId);
            if (previousFolderHasPerFolderSortOrder) {
                this.setPerFolderSortOrder(this.previousFolderId, field, reverse);
            }
            else {
                this.setSharedSortOrder(field, reverse);
            }
        }
        this.previousFolderId = selectedId;
        let next;
        if (this.isSet(selectedId)) {
            next = this.get(selectedId);
        }
        else if (previousFolderHasPerFolderSortOrder) {
            next = this.sharedSortOrder;
        }
        else {
            return;
        }
        if (Setting_1.default.value('notes.perFolderSortOrderEnabled')) {
            if (next.field !== field || next.reverse !== reverse) {
                (0, notesSortOrderUtils_1.setNotesSortOrder)(next.field, next.reverse);
            }
        }
    }
    static getSelectedFolderId() {
        if (this.folderState.notesParentType === 'Folder') {
            return this.folderState.selectedFolderId;
        }
        else if (this.folderState.notesParentType === 'SmartFilter') {
            return this.folderState.selectedSmartFilterId;
        }
        else {
            return '';
        }
    }
    static loadPerFolderSortOrders() {
        this.perFolderSortOrders = Object.assign({}, Setting_1.default.value('notes.perFolderSortOrders'));
    }
    static loadSharedSortOrder() {
        const validFields = (0, notesSortOrderUtils_1.notesSortOrderFieldArray)();
        const value = Setting_1.default.value('notes.sharedSortOrder');
        for (const key in this.sharedSortOrder) {
            if (value.hasOwnProperty(key)) {
                if (key !== 'field' || validFields.includes(value.field)) {
                    this.sharedSortOrder[key] = value[key];
                }
            }
        }
    }
    static setPerFolderSortOrder(folderId, field, reverse) {
        const old = this.get(folderId);
        let dirty = false;
        if (!((old === null || old === void 0 ? void 0 : old.field) === field)) {
            this.perFolderSortOrders[folderId + SUFFIX_FIELD] = field;
            dirty = true;
        }
        if (!((old === null || old === void 0 ? void 0 : old.reverse) === reverse)) {
            this.perFolderSortOrders[folderId + SUFFIX_REVERSE] = reverse;
            dirty = true;
        }
        if (dirty) {
            Setting_1.default.setValue('notes.perFolderSortOrders', Object.assign({}, this.perFolderSortOrders));
        }
    }
    static deletePerFolderSortOrder(folderId) {
        let dirty = false;
        if (this.perFolderSortOrders.hasOwnProperty(folderId + SUFFIX_FIELD)) {
            delete this.perFolderSortOrders[folderId + SUFFIX_FIELD];
            dirty = true;
        }
        if (this.perFolderSortOrders.hasOwnProperty(folderId + SUFFIX_REVERSE)) {
            delete this.perFolderSortOrders[folderId + SUFFIX_REVERSE];
            dirty = true;
        }
        if (dirty) {
            Setting_1.default.setValue('notes.perFolderSortOrders', Object.assign({}, this.perFolderSortOrders));
        }
    }
    static setSharedSortOrder(field, reverse) {
        let dirty = false;
        if (this.sharedSortOrder.field !== field) {
            this.sharedSortOrder.field = field;
            dirty = true;
        }
        if (this.sharedSortOrder.reverse !== reverse) {
            this.sharedSortOrder.reverse = reverse;
            dirty = true;
        }
        if (this.sharedSortOrder[field] !== reverse) {
            this.sharedSortOrder[field] = reverse;
            dirty = true;
        }
        if (dirty) {
            Setting_1.default.setValue('notes.sharedSortOrder', Object.assign({}, this.sharedSortOrder));
        }
    }
}
// To support a custom sort order in "all notebooks", previousFolderId
// can also be a smart filter ID.
PerFolderSortOrderService.previousFolderId = null;
PerFolderSortOrderService.folderState = { notesParentType: '', selectedFolderId: '', selectedSmartFilterId: '' };
// Since perFolderSortOrders and sharedSortOrder is persisted using Setting,
// their structures are not nested.
PerFolderSortOrderService.perFolderSortOrders = null;
PerFolderSortOrderService.sharedSortOrder = {
    field: 'user_updated_time',
    reverse: true,
    user_updated_time: true,
    user_created_time: true,
    title: false,
    order: false,
};
exports.default = PerFolderSortOrderService;
//# sourceMappingURL=PerFolderSortOrderService.js.map