"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const util_1 = require("util");
const githubReleasesUtils_1 = require("./githubReleasesUtils");
const generateLatestArm64Yml_1 = require("./generateLatestArm64Yml");
const basePath = path.join(__dirname, '..');
const downloadDir = path.join(basePath, 'downloads');
// Renames release assets in Joplin Desktop releases
const renameReleaseAssets = async (context, release) => {
    // Patterns used to rename releases
    const renamePatterns = [
        [/-arm64\.dmg$/, '-arm64.DMG'],
    ];
    for (const asset of release.assets) {
        for (const [pattern, replacement] of renamePatterns) {
            if (asset.name.match(pattern)) {
                const newName = asset.name.replace(pattern, replacement);
                await (0, githubReleasesUtils_1.updateReleaseAsset)(context, asset.url, newName);
                asset.name = newName;
                // Only rename a release once.
                break;
            }
        }
    }
};
// Creates release assets in Joplin Desktop releases
const createReleaseAssets = async (context, release) => {
    // Create latest-mac-arm64.yml file and publish
    let dmgPath;
    let zipPath;
    for (const asset of release.assets) {
        console.log(`Checking asset: ${asset.name}`);
        if (asset.name.endsWith('arm64.zip')) {
            zipPath = await (0, githubReleasesUtils_1.downloadFileFromGitHub)(context, asset, downloadDir);
        }
        else if (asset.name.endsWith('arm64.DMG')) {
            dmgPath = await (0, githubReleasesUtils_1.downloadFileFromGitHub)(context, asset, downloadDir);
        }
    }
    if (!zipPath || !dmgPath) {
        const formattedAssets = release.assets.map(asset => ({
            name: asset.name,
            url: asset.url,
        }));
        throw new Error(`Zip path: ${zipPath} and/or dmg path: ${dmgPath} are not defined. Logging assets of release: ${JSON.stringify(formattedAssets, null, 2)}`);
    }
    const info = {
        version: release.tag_name.slice(1),
        dmgPath: dmgPath,
        zipPath: zipPath,
        releaseDate: new Date().toISOString(),
    };
    const latestArm64FilePath = (0, generateLatestArm64Yml_1.generateLatestArm64Yml)(info, downloadDir);
    await (0, githubReleasesUtils_1.uploadReleaseAsset)(context, release, latestArm64FilePath);
};
const modifyReleaseAssets = async () => {
    const args = (0, util_1.parseArgs)({
        options: {
            tag: { type: 'string' },
            token: { type: 'string' },
            repo: { type: 'string' },
        },
    });
    if (!args.values.tag || !args.values.token || !args.values.repo) {
        throw new Error([
            'Required arguments: --tag, --token, --repo',
            '  --tag should be a git tag with an associated release (e.g. v12.12.12)',
            '  --token should be a GitHub API token',
            '  --repo should be a string in the form user/reponame (e.g. laurent22/joplin)',
        ].join('\n'));
    }
    const context = {
        repo: args.values.repo,
        githubToken: args.values.token,
        targetTag: args.values.tag,
    };
    const release = await (0, githubReleasesUtils_1.getTargetRelease)(context, context.targetTag);
    if (!release.assets) {
        console.log(release);
        throw new Error(`Release ${release.tag_name} missing assets!`);
    }
    console.log('Renaming release assets for tag', context.targetTag, context.repo);
    await renameReleaseAssets(context, release);
    console.log('Creating latest-mac-arm64.yml asset for tag', context.targetTag, context.repo);
    await createReleaseAssets(context, release);
};
void modifyReleaseAssets();
//# sourceMappingURL=modifyReleaseAssets.js.map