"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const electron_1 = require("electron");
const path_1 = require("path");
const url_1 = require("url");
const constants_1 = require("./constants");
const resolvePathWithinDir_1 = require("@joplin/lib/utils/resolvePathWithinDir");
const fs = require("fs-extra");
const fs_1 = require("fs");
const mime_utils_1 = require("@joplin/lib/mime-utils");
const uuid_1 = require("@joplin/lib/uuid");
// In some cases, the NodeJS built-in adapter (Readable.toWeb) closes its controller twice,
// leading to an error dialog. See:
// - https://github.com/nodejs/node/blob/e578c0b1e8d3dd817e692a0c5df1b97580bc7c7f/lib/internal/webstreams/adapters.js#L454
// - https://github.com/nodejs/node/issues/54205
// We work around this by creating a more-error-tolerant custom adapter.
const nodeStreamToWeb = (resultStream) => {
    resultStream.pause();
    let closed = false;
    return new ReadableStream({
        start: (controller) => {
            resultStream.on('data', (chunk) => {
                if (closed) {
                    return;
                }
                if (Buffer.isBuffer(chunk)) {
                    controller.enqueue(new Uint8Array(chunk));
                }
                else {
                    controller.enqueue(chunk);
                }
                if (controller.desiredSize <= 0) {
                    resultStream.pause();
                }
            });
            resultStream.on('error', (error) => {
                controller.error(error);
            });
            resultStream.on('end', () => {
                if (!closed) {
                    closed = true;
                    controller.close();
                }
            });
        },
        pull: (_controller) => {
            if (closed) {
                return;
            }
            resultStream.resume();
        },
        cancel: () => {
            if (!closed) {
                closed = true;
                resultStream.close();
            }
        },
    }, { highWaterMark: resultStream.readableHighWaterMark });
};
// Allows seeking videos.
// See https://github.com/electron/electron/issues/38749 for why this is necessary.
const handleRangeRequest = async (request, targetPath) => {
    var _a, _b;
    const makeUnsupportedRangeResponse = () => {
        return new Response('unsupported range', {
            status: 416, // Range Not Satisfiable
        });
    };
    const rangeHeader = request.headers.get('Range');
    if (!rangeHeader.startsWith('bytes=')) {
        return makeUnsupportedRangeResponse();
    }
    const stat = await fs.stat(targetPath);
    // Ranges are requested using one of the following formats
    //  bytes=1234-5679
    //  bytes=-5678
    //  bytes=1234-
    // See https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Range
    const startByte = Number(((_a = rangeHeader.match(/(\d+)-/)) === null || _a === void 0 ? void 0 : _a[1]) || '0');
    const endByte = Number(((_b = rangeHeader.match(/-(\d+)/)) === null || _b === void 0 ? void 0 : _b[1]) || `${stat.size - 1}`);
    if (endByte > stat.size || startByte < 0) {
        return makeUnsupportedRangeResponse();
    }
    // Note: end is inclusive.
    const resultStream = (0, fs_1.createReadStream)(targetPath, { start: startByte, end: endByte });
    // See the HTTP range requests guide: https://developer.mozilla.org/en-US/docs/Web/HTTP/Range_requests
    const headers = new Headers([
        ['Accept-Ranges', 'bytes'],
        ['Content-Type', (0, mime_utils_1.fromFilename)(targetPath)],
        ['Content-Length', `${endByte + 1 - startByte}`],
        ['Content-Range', `bytes ${startByte}-${endByte}/${stat.size}`],
    ]);
    return new Response(nodeStreamToWeb(resultStream), { headers, status: 206 });
};
// Creating a custom protocol allows us to isolate iframes by giving them
// different domain names from the main Joplin app.
//
// For example, an iframe with url joplin-content://note-viewer/path/to/iframe.html will run
// in a different process from a parent frame with url file://path/to/iframe.html.
//
// See note_viewer_isolation.md for why this is important.
//
// TODO: Use Logger.create (doesn't work for now because Logger is only initialized
// in the main process.)
const handleCustomProtocols = (logger) => {
    logger = Object.assign(Object.assign({}, logger), { debug: () => { } });
    // Allow-listed files/directories for joplin-content://note-viewer/
    const readableDirectories = [];
    const readableFiles = new Map();
    // Access for joplin-content://file-media/
    let mediaAccessKey = false;
    // See also the protocol.handle example: https://www.electronjs.org/docs/latest/api/protocol#protocolhandlescheme-handler
    electron_1.protocol.handle(constants_1.contentProtocolName, async (request) => {
        const url = new URL(request.url);
        const host = url.host;
        let pathname = (0, path_1.normalize)((0, url_1.fileURLToPath)(`file://${url.pathname}`));
        // See https://security.stackexchange.com/a/123723
        if (pathname.startsWith('..')) {
            throw new Error(`Invalid URL (not absolute), ${request.url}`);
        }
        pathname = (0, path_1.resolve)(appBundleDirectory, pathname);
        let canRead = false;
        let mediaOnly = true;
        if (host === 'note-viewer') {
            if (readableFiles.has(pathname)) {
                canRead = true;
            }
            else {
                for (const readableDirectory of readableDirectories) {
                    if ((0, resolvePathWithinDir_1.default)(readableDirectory, pathname)) {
                        canRead = true;
                        break;
                    }
                }
            }
            mediaOnly = false;
        }
        else if (host === 'file-media') {
            if (!mediaAccessKey) {
                throw new Error('Media access denied. This must be enabled with .setMediaAccessEnabled');
            }
            canRead = true;
            mediaOnly = true;
            const accessKey = url.searchParams.get('access-key');
            if (accessKey !== mediaAccessKey) {
                throw new Error(`Invalid or missing media access key (was ${accessKey}). An allow-listed ?access-key= parameter must be provided.`);
            }
        }
        else {
            throw new Error(`Invalid URL ${request.url}`);
        }
        if (!canRead) {
            throw new Error(`Read access not granted for URL ${request.url}`);
        }
        const asFileUrl = (0, url_1.pathToFileURL)(pathname).toString();
        logger.debug('protocol handler: Fetch file URL', asFileUrl);
        const rangeHeader = request.headers.get('Range');
        let response;
        if (!rangeHeader) {
            response = await electron_1.net.fetch(asFileUrl);
        }
        else {
            response = await handleRangeRequest(request, pathname);
        }
        if (mediaOnly) {
            // Tells the browser to avoid MIME confusion attacks. See
            // https://blog.mozilla.org/security/2016/08/26/mitigating-mime-confusion-attacks-in-firefox/
            response.headers.set('X-Content-Type-Options', 'nosniff');
            // This is an extra check to prevent loading text/html and arbitrary non-media content from the URL.
            const contentType = response.headers.get('Content-Type');
            if (!contentType || !contentType.match(/^(image|video|audio)\//)) {
                throw new Error(`Attempted to access non-media file from ${request.url}, which is media-only. Content type was ${contentType}.`);
            }
        }
        return response;
    });
    const appBundleDirectory = (0, path_1.dirname)((0, path_1.dirname)(__dirname));
    return {
        allowReadAccessToDirectory: (path) => {
            path = (0, path_1.resolve)(appBundleDirectory, path);
            logger.debug('protocol handler: Allow read access to directory', path);
            readableDirectories.push(path);
        },
        allowReadAccessToFile: (path) => {
            path = (0, path_1.resolve)(appBundleDirectory, path);
            logger.debug('protocol handler: Allow read access to file', path);
            if (readableFiles.has(path)) {
                readableFiles.set(path, readableFiles.get(path) + 1);
            }
            else {
                readableFiles.set(path, 1);
            }
            return {
                remove: () => {
                    var _a;
                    if (((_a = readableFiles.get(path)) !== null && _a !== void 0 ? _a : 0) <= 1) {
                        logger.debug('protocol handler: Remove read access to file', path);
                        readableFiles.delete(path);
                    }
                    else {
                        readableFiles.set(path, readableFiles.get(path) - 1);
                    }
                },
            };
        },
        setMediaAccessEnabled: (enabled) => {
            if (enabled) {
                mediaAccessKey || (mediaAccessKey = (0, uuid_1.createSecureRandom)());
            }
            else {
                mediaAccessKey = false;
            }
        },
        // Allows access to all local media files, provided a matching ?access-key=<key> is added
        // to the request URL.
        getMediaAccessKey: () => {
            return mediaAccessKey || null;
        },
    };
};
exports.default = handleCustomProtocols;
//# sourceMappingURL=handleCustomProtocols.js.map